// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.datasource.wizard;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;

/**
 * <p>CdmDataSourceMySQLWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created 19.05.2009
 * @version 1.0
 */
public class CdmDataSourceMySQLWizardPage extends CdmDataSourceCredentialsWizardPage {
	
	private Text text_port;
	private Text text_server;

	private String server;

	private int port;

	/**
	 * <p>Constructor for CdmDataSourceMySQLWizardPage.</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 */
	protected CdmDataSourceMySQLWizardPage(ICdmDataSource dataSource) {
		super("MySQL");
		setTitle("MySQL Server");
		setDescription("Enter credentials for MySQL database");
		this.setDataSource(dataSource);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.datasource.CdmDataSourceCredentialsWizardPage#createDatabaseForm()
	 */
	/** {@inheritDoc} */
	@Override
	public void createDatabaseForm() {
		// Create group composite for location data 
		locationGroup = new Group(composite, SWT.NONE);
		locationGroup.setText("Location");
		locationGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		GridLayout locationLayout = new GridLayout();
		locationLayout.numColumns = 2;
		locationGroup.setLayout(locationLayout);

		// Create host label
		Label serverLabel = new Label(locationGroup, SWT.NONE);
		serverLabel.setText("Host:");

		// Create host input
		text_server = new Text(locationGroup, SWT.BORDER);
		text_server.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text_server.addModifyListener(this);

		// Create port label
		Label portLabel = new Label(locationGroup, SWT.NONE);
		portLabel.setText("Port:");

		// Create port input
		text_port = new Text(locationGroup, SWT.BORDER);
		text_port.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		text_port.addModifyListener(this);
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.datasource.CdmDataSourceCredentialsWizardPage#updateLocation()
	 */
	/** {@inheritDoc} */
	@Override
	public void updateLocation() {
		server = text_server.getText();
		try{
			if(! text_port.getText().equals("")){
				port = new Integer(text_port.getText());
				setErrorMessage(null);
			}
		}catch(NumberFormatException e){
			setErrorMessage("Port number contains invalid characters");
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceCredentialsWizardPage#updateDataSource()
	 */
	/** {@inheritDoc} */
	@Override
	public void updateDataSource() {
		setDataSource(CdmDataSource.NewMySqlInstance(server,
													database,
													port,
													username,
													password,
													nomenclaturalCode));
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceCredentialsWizardPage#checkPageComplete()
	 */
	/** {@inheritDoc} */
	@Override
	public void checkPageComplete() {
		// check if widgets of this component are complete
		boolean complete = server.trim().length() != 0;
		
		// set default port
		if(port == 0){
			port = DatabaseTypeEnum.MySQL.getDefaultPort();
		}
		
		setPageComplete(complete);
		
		// check if widgets of the extended class are complete
		super.checkPageComplete();
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceCredentialsWizardPage#init()
	 */
	/** {@inheritDoc} */
	@Override
	public void init() {
		if(getDataSource() != null){
			modifyTextWithoutTriggeringListeners(text_server, this, getDataSource().getServer());
			modifyTextWithoutTriggeringListeners(text_port, this, getDataSource().getPort()+"");			
			super.init();
		}
	}
}
