// $Id$
/**
 * Copyright (C) 2009 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.definedterm;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.definedterm.input.AbstractDefinedTermEditorInput;
import eu.etaxonomy.taxeditor.model.IDirtyMarkableSelectionProvider;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * @author l.morris
 * @date 3 Jan 2012
 *
 */
public class DefinedTermEditor extends EditorPart implements IConversationEnabled, ISelectionListener, IDirtyMarkableSelectionProvider, IPostOperationEnabled {

	public static final String ID = "eu.etaxonomy.taxeditor.editor.definedTerm";

	public static final String OPEN_COMMAND_ID = "eu.etaxonomy.taxeditor.store.openDefinedTermEditor";
	
	protected TreeViewer viewer;
	protected ConversationHolder conversation;
	private ISelectionService selectionService;
	private boolean dirty;
	Set<TermVocabulary<? extends DefinedTermBase>> inMemoryVocabularies = new HashSet<TermVocabulary<? extends DefinedTermBase>>();

	private int dndOperations = DND.DROP_COPY | DND.DROP_MOVE;
	/**
	 * 
	 */
	public DefinedTermEditor() {
		super();
		conversation = CdmStore.createConversation();
	}

	@Override
	public void createPartControl(Composite parent) {
			
	//		parent.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_BLUE));
			
			FillLayout layout = new FillLayout();
			layout.marginHeight = 0;
			layout.marginWidth = 0;
			layout.type = SWT.VERTICAL;
			
			parent.setLayout(layout);
			viewer = new TreeViewer(parent);
			viewer.getControl().setLayoutData(LayoutConstants.FILL());
			viewer.setContentProvider(new TermContentProvider());
			viewer.setLabelProvider(new TermLabelProvider());
			viewer.setSorter(new ViewerSorter());
			
			Transfer[] transfers = new Transfer[] { TermTransfer.getInstance() };
			viewer.addDragSupport(dndOperations, transfers, new DefinedTermDragListener(viewer));
			viewer.addDropSupport(dndOperations, transfers, new DefinedTermDropAdapter(this));

					
			initialiseVocabularies();		
			viewer.setInput(inMemoryVocabularies);
						
			getSite().setSelectionProvider(viewer);
			
			selectionService = getSite().getWorkbenchWindow().getSelectionService();
			selectionService.addSelectionListener(this);
	
			setPartName(getEditorInput().getName());
			createMenu();
		}
	
	
	protected void initialiseVocabularies() {
		inMemoryVocabularies.clear();
		//List<TermVocabulary> vocabularies = CdmStore.getService(IVocabularyService.class).list(null, null, null, null, null);

//		for (TermVocabulary vocabulary : vocabularies){
//			if(vocabulary.getTerms().isEmpty()){
//				inMemoryVocabularies.add(vocabulary);
//			}
//		}

		
		List<TermVocabulary<? extends DefinedTermBase>> termVocabularies = CdmStore.getService(IVocabularyService.class).listByTermClass(getDefinedTermEditorInput().getTermClass(), true, true, null, null, null, null);
		
		
		inMemoryVocabularies.addAll(termVocabularies);
	}
	
	/**
	 * 
	 */
	public AbstractDefinedTermEditorInput getDefinedTermEditorInput() {
		return (AbstractDefinedTermEditorInput) getEditorInput();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {

		if(objectAffectedByOperation instanceof DefinedTermBase){
			viewer.refresh();
		} else if (objectAffectedByOperation instanceof TermVocabulary){
//			if (inMemoryVocabularies.contains(objectAffectedByOperation)){
//				inMemoryVocabularies.remove((TermVocabulary) objectAffectedByOperation);
//			}else{
//				inMemoryVocabularies.add((TermVocabulary) objectAffectedByOperation);
//				DetailsViewPart view = (DetailsViewPart) StoreUtil.getView(DetailsViewPart.ID, true);
//				view.setFocus();
//			}
			
//			viewer.setInput(inMemoryVocabularies);
			viewer.refresh();
			
		}
		
		
		
		if(objectAffectedByOperation != null){
			viewer.setSelection(new StructuredSelection(objectAffectedByOperation));
		}
		setDirty(true);

		return true;
	} 

	private void createMenu() {
		// register context menu
		MenuManager menuManager = new MenuManager();
		menuManager
				.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(ID, menuManager, viewer);
	
		Control control = viewer.getControl();
		Menu menu = menuManager.createContextMenu(control);
		control.setMenu(menu);
	}
	

		

	@Override
	public void doSave(IProgressMonitor monitor) {
		getConversationHolder().commit();
		setDirty(false);
		initialiseVocabularies();
	}
	

	@Override
	public void doSaveAs() {}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		setSite(site);
		setInput(input);
		
	}

	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	@Override
	public void setFocus() {
		getConversationHolder().bind();
	}

	@Override
	public void update(CdmDataChangeMap changeEvents) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void changed(Object element) {
		dirty = true;
		setDirty(true);
		viewer.update(element, null);
	}

	/**
	 * @param dirty the dirty to set
	 */
	public void setDirty(boolean dirty) {
		this.dirty = dirty;
		firePropertyChange(PROP_DIRTY);
	}

	@Override
	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * 
	 */
	public TreeViewer getViewer() {
		return viewer;
	}

	/**
	 * @return 
	 * 
	 */
	public Set<TermVocabulary<? extends DefinedTermBase>> getInMemoryVocabularies() {
		return inMemoryVocabularies;
	}

}