// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.definedterm.operation;

import java.util.Collection;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author l.morris
 * @date 10 Jan 2012
 *
 */
public class MoveDefinedTermOperation extends AbstractPostOperation {
	
	private Collection<DefinedTermBase> sourceTerms;// the actual DefinedTermBase(s) we are moving
	private TermBase targetTermOrVocabulary;// the target VOCABULARY or DefinedTerm we are moving these to

	/**
	 * @param label
	 * @param undoContext
	 * @param postOperationEnabled
	 */
	public MoveDefinedTermOperation(String label, IUndoContext undoContext, TermBase target, Collection<DefinedTermBase> sourceTerms,
			IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		
		this.targetTermOrVocabulary = target;
		this.sourceTerms = sourceTerms;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// need to make the moved DefinedTerm part of another DefinedTerm or Vocabulary (target)
		// and remove it from old associations
		
		//TODO move to ITermService
		
		for (DefinedTermBase term : sourceTerms){
			// do nothing when moving it on itself
			if(targetTermOrVocabulary.equals(term)){
				Status status = new Status(IStatus.CANCEL, StoreUtil.getPluginId(), "Term can not be added to itself");
				StoreUtil.informationDialog("", status);
				return status;
			}
			
			if (targetTermOrVocabulary instanceof TermVocabulary) {
				TermVocabulary termVocabulary = (TermVocabulary)targetTermOrVocabulary;
				
				// do nothing when term is top level and gets added to the same vocabulary
				if(term.getPartOf() == null && termVocabulary.equals(term.getVocabulary())){
					Status status = new Status(IStatus.CANCEL, StoreUtil.getPluginId(), "Term is already in this vocabulary");
					StoreUtil.informationDialog("", status);
					return status;
				}
				
				cleanTerm(term);
				termVocabulary.addTerm(term);
				
			} else if (targetTermOrVocabulary instanceof DefinedTermBase) {
				cleanTerm(term);
				DefinedTermBase targetDefinedTerm = (DefinedTermBase) targetTermOrVocabulary;
				targetDefinedTerm.addIncludes(term);
				targetDefinedTerm.getVocabulary().addTerm(term);				
			}

		}
		//return null;
		return postExecute(targetTermOrVocabulary);			
	}
	
	private DefinedTermBase cleanTerm(DefinedTermBase term){

		DefinedTermBase partOf = term.getPartOf();
		if(partOf != null){
			partOf.removeIncludes(term);
		}
	
		
		TermVocabulary vocabulary = term.getVocabulary();
		if(vocabulary != null){
			vocabulary.removeTerm(term);
		}
		
		return term;
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}

}
