/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.preference;

import java.util.List;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * <p>
 * NomenclaturalCodePreferences class.
 * </p>
 * 
 * @author p.ciardelli
 * @created 16.09.2008
 * @version 1.0
 */
public class NomenclaturalCodePreferences extends FieldEditorPreferencePage
		implements IWorkbenchPreferencePage {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors
	 * ()
	 */
	@Override
	protected void createFieldEditors() {

		addField(new RadioGroupFieldEditor(
				IPreferenceKeys.PREFERRED_NOMENCLATURAL_CODE_KEY,
				"Available Codes", 1, getLabelAndValues(),
				getFieldEditorParent(), true));

	}

	/**
	 * @return
	 */
	private String[][] getLabelAndValues() {
		List<NomenclaturalCode> supportedCodes = NomenclaturalCodeHelper
				.getSupportedCodes();
		String[][] labelAndValues = new String[supportedCodes.size()][2];
		for (int i = 0; i < supportedCodes.size(); i++) {
			labelAndValues[i][0] = NomenclaturalCodeHelper
					.getDescription(supportedCodes.get(i));
			labelAndValues[i][1] = PreferencesUtil
					.getPreferenceKey(supportedCodes.get(i));
		}
		return labelAndValues;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * Initialize the preference page
	 */
	public void init(IWorkbench workbench) {
		setPreferenceStore(TaxeditorStorePlugin.getDefault()
				.getPreferenceStore());
		setDescription("Choose which nomenclatural code you would like to use for scientific names unless otherwise specified.");
	}

}
