// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.wizard;


import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Representation;

/**
 * <p>VocabularyTermWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created 12.06.2009
 * @version 1.0
 */
public class VocabularyTermWizardPage<T extends DefinedTermBase> extends WizardPage implements ModifyListener {

	private Text text_label;
	private Text text_labelAbbreviation;
	private Text text_description;
	
	/**
	 * <p>Constructor for VocabularyTermWizardPage.</p>
	 */
	public VocabularyTermWizardPage() {
		super("VOCABULARY_TERM_WIZARD_PAGE");		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	public void createControl(Composite parent) {
		
		String termLabel = getWizard().getTermClass().getSimpleName();
		
		String title = getWizard().isEditMode() ? "Edit " + termLabel : "Create new " + termLabel;
		this.setTitle(title);
		
		Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.horizontalSpacing = SWT.FILL;
		gridLayout.verticalSpacing = SWT.FILL;
		composite.setLayout(gridLayout);
		
		// label
		CLabel label_label = new CLabel(composite, SWT.NULL);
		label_label.setText("Label");
		
		text_label = new Text(composite, SWT.BORDER);
		text_label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_label.addModifyListener(this);
		
		// label abbreviation 
		CLabel label_labelAbbreviation = new CLabel(composite, SWT.NULL);
		label_labelAbbreviation.setText("Label Abbreviation");
		
		text_labelAbbreviation = new Text(composite, SWT.BORDER);
		text_labelAbbreviation.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
		text_labelAbbreviation.addModifyListener(this);
		
		// descriptive text
		CLabel label_description = new CLabel(composite, SWT.NULL);
		label_description.setText("Description");
		label_description.setLayoutData(new GridData(SWT.TOP));
		
		text_description = new Text(composite, SWT.BORDER | SWT.MULTI);
		text_description.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		text_description.addModifyListener(this);
		
		setControl(composite);
		
		if(getWizard().getRepresentation() != null){
			Representation representation = getWizard().getRepresentation();
			text_label.setText(CdmUtils.Nz(representation.getLabel()));
			text_labelAbbreviation.setText(CdmUtils.Nz(representation.getAbbreviatedLabel()));
			text_description.setText(CdmUtils.Nz(representation.getDescription()));
		}
		
		setPageComplete(false);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	/** {@inheritDoc} */
	public void modifyText(ModifyEvent e) {
		Object eventSource = e.getSource();
		if(eventSource == text_label){
			getWizard().getRepresentation().setLabel(text_label.getText());
		}
		else if(eventSource == text_labelAbbreviation){
			getWizard().getRepresentation().setAbbreviatedLabel(text_labelAbbreviation.getText());
		}
		else if(eventSource == text_description){
			getWizard().getRepresentation().setText(text_description.getText());
		}  
		
		
		if(text_label.getText().length() > 0){
			setPageComplete(true);
		}
		else {
			setPageComplete(false);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#getWizard()
	 */
	/** {@inheritDoc} */
	@Override
	public VocabularyTermWizard getWizard() {
		return (VocabularyTermWizard) super.getWizard();
	}
	
	
}
