// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.IGroupService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.config.IFindTaxaAndNamesConfigurator;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.RelationshipBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created Dec 8, 2010
 * @version 1.0
 */
public class SearchManager {
	
	public static final List NO_RESULTS = Arrays.asList(new Object[]{});
	
	public static final String WILDCARD = "*";
	
	public static int NO_COUNT = -1;
	
	// TODO make this configurable via preferences
	private static final int MAX_RESULTS_BEFORE_WARNING = 500; 
	
	/**
	 * <p>findNames</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<TaxonNameBase> findNames(IIdentifiableEntityServiceConfigurator configurator){
		if(checkLargeResult(CdmStore.getService(INameService.class).countByTitle(configurator))){
			return CdmStore.getService(INameService.class).findByTitle(configurator).getRecords();
		}
		return NO_RESULTS;
	}
	
	public List<NameRelationship> findNameRelationships(
			IIdentifiableEntityServiceConfigurator configurator) {
		if(true){
			return NO_RESULTS;
		}
		
		List<NameRelationship> relationships = new ArrayList<NameRelationship>(); 
		List<RelationshipBase> all = CdmStore.getService(INameService.class).getAllRelationships(0, 0);
		
		for (RelationshipBase relationship : all){
			if(relationship instanceof NameRelationship){
				relationships.add((NameRelationship) relationship);
			}
		}
		
		
		return relationships;
		
	}

	/**
	 * <p>findTaxaAndNames</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<UuidAndTitleCache<TaxonBase>> findTaxaAndNames(IFindTaxaAndNamesConfigurator<TaxonBase> configurator){
		return CdmStore.getService(ITaxonService.class).findTaxaAndNamesForEditor(configurator);
	}
	
	
	/**
	 * <p>findReferences</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<Reference> findReferences(IIdentifiableEntityServiceConfigurator configurator){
		if(checkLargeResult(CdmStore.getService(IReferenceService.class).countByTitle(configurator))){
			return  CdmStore.getService(IReferenceService.class).findByTitle(configurator).getRecords();
		}
		return NO_RESULTS;
	}
	
	
	/**
	 * <p>findAgents</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<AgentBase> findAgents(IIdentifiableEntityServiceConfigurator configurator){
		if(checkLargeResult(CdmStore.getService(IAgentService.class).countByTitle(configurator))){
			return CdmStore.getService(IAgentService.class).findByTitle(configurator).getRecords();
		}
		return NO_RESULTS;
	}

	/**
	 * <p>findTeamOrPersons</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<AgentBase> findTeamOrPersons(IIdentifiableEntityServiceConfigurator configurator){
		configurator.setClazz(TeamOrPersonBase.class);
		return findAgents(configurator);
	}
	
	
	/**
	 * <p>findOccurrences</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	public List<SpecimenOrObservationBase> findOccurrences(IIdentifiableEntityServiceConfigurator configurator){
		// by default we do not show field observations. This may be configured via preferences
		boolean showFieldObservations = PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.BULK_EDITOR_OCCURRENCE_SHOW_FIELD_OBSERVATIONS);
		if(! showFieldObservations){
			configurator.setClazz(DerivedUnitBase.class);
		}
		if(checkLargeResult(CdmStore.getService(IOccurrenceService.class).countByTitle(configurator))){
			return CdmStore.getService(IOccurrenceService.class).findByTitle(configurator).getRecords();
		}
		return NO_RESULTS;
	}
	
	
	public List<User> findUsers(IIdentifiableEntityServiceConfigurator configurator){
		String userNameSearchString = sqlizeTitleSearchString(configurator);
		// TODO why are users not identifiable entities?
		return CdmStore.getService(IUserService.class).listByUsername(userNameSearchString, null, null, null, null, null, null);
	}
	
	
	public List<Group> findGroups(IIdentifiableEntityServiceConfigurator configurator){
		String groupNameSearchString = sqlizeTitleSearchString(configurator);
		// TODO why are groups not identifiable entities?
		return CdmStore.getService(IGroupService.class).listByName(groupNameSearchString, null, null, null, null, null, null);
	}
	
	
	/**
	 * @param count
	 * @return
	 */
	private boolean checkLargeResult(int count) {
		if(count > MAX_RESULTS_BEFORE_WARNING){
			return MessageDialog.openConfirm(Display.getDefault().getActiveShell(), "Large result expected", 
					String.format("The current search will return %s objects. This will " +
							"take a long time and/or might render the editor unusable. Please consider refining your search.", count));
		}else{
			return true;
		}		
	}
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	private String sqlizeTitleSearchString(IIdentifiableEntityServiceConfigurator configurator){
		return configurator.getTitleSearchString().replace(WILDCARD, "%");
	}


}
