// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>UriDialog class.</p>
 *
 * @author p.ciardelli
 * @created 31.03.2009
 * @version 1.0
 *
 * TODO make this extend TitleAreaDialog
 */
public class UriDialog extends InputDialog {
		
	/**
	 * @param parentShell
	 * @param dialogTitle
	 * @param dialogMessage
	 * @param initialValue
	 * @param validator
	 */
	private UriDialog(Shell parentShell, String dialogTitle,
			String dialogMessage) {
		super(parentShell, dialogTitle, 
				dialogMessage, "http://", new IInputValidator() {

					public String isValid(String text) {
						
						try {
							new URI(text);
						} catch (URISyntaxException e) {
							return "URL not correctly formed.";
						}
						
						
						if (text.length() <= "http://".length()) {
							return "";
						}
						return null;
					}
			
		});
	}

	private URI getUri() {
		URI uri = null;
		try {
			uri = new URI(getValue());
		} catch (URISyntaxException e) {
			StoreUtil.error(getClass(), e);
		}
		return uri;
	}
	
	/**
	 * <p>getUri</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param dialogTitle a {@link java.lang.String} object.
	 * @param dialogMessage a {@link java.lang.String} object.
	 * @return a {@link java.net.URI} object.
	 */
	public static URI getUri(Shell shell, String dialogTitle,
			String dialogMessage) {
		UriDialog dialog = new UriDialog(shell, 
				dialogTitle, 
				dialogMessage);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		return dialog.getUri();
	}
}
