// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IFeatureTreeService;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.taxeditor.featuretree.FeatureTreeEditorWizard;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>FilteredFeatureTreeSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created Sep 17, 2010
 * @version 1.0
 */
public class FeatureTreeSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<FeatureTree> {
	
	/**
	 * Creates a filtered selection dialog to select a named area.
	 *
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param featureTree a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 * @return a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	public static FeatureTree select(Shell shell, ConversationHolder conversation, FeatureTree featureTree) {
		FeatureTreeSelectionDialog dialog = new FeatureTreeSelectionDialog(shell, conversation,
				"Choose a feature tree", false, featureTree);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	 * <p>Constructor for FilteredFeatureTreeSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param cdmObject a {@link eu.etaxonomy.cdm.model.description.FeatureTree} object.
	 */
	protected FeatureTreeSelectionDialog(Shell shell,
			ConversationHolder conversation, String title, boolean multi,
			FeatureTree cdmObject) {
		super(shell, conversation, title, multi, FeatureTreeSelectionDialog.class.getCanonicalName(), cdmObject);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	/** {@inheritDoc} */
	@Override
	protected FeatureTree getPersistentObject(UUID uuid) {
		return CdmStore.getService(IFeatureTreeService.class).load(uuid);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	/** {@inheritDoc} */
	@Override
	protected void initModel() {
		List<FeatureTree> featureTrees = CdmStore.getService(IFeatureTreeService.class).list(FeatureTree.class, null, null, null, null);
		
		if(model == null){
			model = new ArrayList<UuidAndTitleCache<FeatureTree>>();
		}
		model.clear();
		for(FeatureTree featureTree : featureTrees){
			UuidAndTitleCache<FeatureTree> element = new UuidAndTitleCache<FeatureTree>(FeatureTree.class, featureTree.getUuid(), featureTree.getTitleCache()); 
			model.add(element);
		}
	}

	/* (non-Javadoc)
	* @see org.eclipse.ui.dialogs.FilteredItemsSelectionDialog#createExtendedContentArea(org.eclipse.swt.widgets.Composite)
	*/
	/** {@inheritDoc} */
	@Override
	protected Control createExtendedContentArea(Composite parent) {
		Link link = new Link(parent, SWT.NONE);
		link.setText(getNewWizardLinkText());
		link.addListener (SWT.Selection, new Listener () {
			public void handleEvent(Event event) {
				
				FeatureTreeEditorWizard wizard = new FeatureTreeEditorWizard();
				WizardDialog dialog = new WizardDialog(getShell(), wizard);
				int status = dialog.open();
				
				if (status == IStatus.OK) {
					refresh();
				}
			}
		});
		return link;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	/** {@inheritDoc} */
	@Override
	protected String getNewWizardLinkText() {
		return "Click <A>here</A> to create or edit a feature tree.";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewEntityWizard()
	 */
	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return null;
	}
}
