// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.dialog.selection;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.occurrence.FieldObservation;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @date Jan 25, 2012
 *
 */
public class SelectionDialogFactory {

	public static <T extends ICdmBase> T getSelectionFromDialog(Class<T> clazz, Shell shell, ConversationHolder conversation, T curentSelection){
		
		if(clazz.equals(Taxon.class)){
			return (T) TaxonBaseSelectionDialog.selectTaxon(shell, conversation, (Taxon) curentSelection);
		}
		if(clazz.equals(Synonym.class)){
			return (T) TaxonBaseSelectionDialog.selectSynonym(shell, conversation);
		}
		if(clazz.equals(TaxonBase.class)){
			return (T) TaxonBaseSelectionDialog.selectTaxonBase(shell, conversation);
		}
		if(clazz.equals(Classification.class)){
			return (T) ClassificationSelectionDialog.select(shell, conversation, (Classification) curentSelection);
		}
		if(clazz.equals(TaxonNode.class)){
			return (T) TaxonNodeSelectionDialog.select(shell, conversation, null, null, (TaxonNode) curentSelection, null);
		}
		if(clazz.equals(Reference.class)){
			return (T) ReferenceSelectionDialog.select(shell, conversation, (Reference) curentSelection);
		}
		if(clazz.equals(TaxonNameBase.class)){
			return (T) NameSelectionDialog.select(shell, conversation, (TaxonNameBase) curentSelection);
		}
		if(clazz.equals(Team.class)){
			return (T) TeamSelectionDialog.select(shell, conversation, (Team) curentSelection);
		}
		if(clazz.equals(AgentBase.class)){
			return (T) AgentSelectionDialog.select(shell, conversation, (AgentBase) curentSelection);
		}
		if(clazz.equals(Feature.class)){
			return (T) FeatureSelectionDialog.select(shell, conversation, (Feature) curentSelection);
		}
		if(clazz.equals(FeatureTree.class)){
			return (T) FeatureTreeSelectionDialog.select(shell, conversation, (FeatureTree) curentSelection);
		}
		if(clazz.equals(PolytomousKey.class)){
			return (T) PolytomousKeySelectionDialog.select(shell, conversation, (PolytomousKey) curentSelection);
		}
		if(clazz.equals(PolytomousKeyNode.class)){
			StoreUtil.warningDialog("Not implemented yet", SelectionDialogFactory.class,
					"This functionality is not implemented yet.");
			return null;
		}
		if(clazz.equals(DerivedUnitBase.class)){
			return (T) DerivedUnitSelectionDialog.select(shell, conversation, (DerivedUnitBase) curentSelection);
		}
		if(clazz.equals(FieldObservation.class)){
			return (T) FieldObservationSelectionDialog.select(shell, conversation, (FieldObservation) curentSelection);
		}
		if(clazz.equals(NamedArea.class)){
			return (T) NamedAreaSelectionDialog.select(shell, conversation, (NamedArea) curentSelection);
		}
		if(clazz.equals(Collection.class)){
			return (T) CollectionSelectionDialog.select(shell, conversation, (Collection) curentSelection);
		}
		if(clazz.equals(User.class)){
			return (T) UserSelectionDialog.select(shell, conversation, (User) curentSelection);
		}
		if(clazz.equals(GrantedAuthorityImpl.class)){
			return (T) GrantedAuthoritySelectionDialog.select(shell, conversation, (GrantedAuthorityImpl) curentSelection);
		}
		if (clazz.equals(Person.class)){
			return (T) PersonSelectionDialog.select(shell, conversation, (Person) curentSelection);
		}
		if(clazz.equals(Group.class)){
			return (T) GroupSelectionDialog.select(shell, conversation, (Group) curentSelection);
		}
		if(clazz.equals(Institution.class)){
			return (T) InstitutionSelectionDialog.select(shell, conversation, (Institution) curentSelection);
		}
				
		return null;
	}
	
}
