// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewUserWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class UserSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<User> {

	/**
	 * 
	 * @param shell
	 * @param conversation
	 * @param user
	 * @return
	 */
	public static User select(Shell shell, ConversationHolder conversation, User user){
		UserSelectionDialog dialog = new UserSelectionDialog(shell, conversation,
				"Choose User", false, UserSelectionDialog.class.getCanonicalName(), user);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	 * @param shell
	 * @param conversation
	 * @param title
	 * @param multi
	 * @param settings
	 * @param cdmObject
	 */
	protected UserSelectionDialog(Shell shell, ConversationHolder conversation,
			String title, boolean multi, String settings, User cdmObject) {
		super(shell, conversation, title, multi, settings, cdmObject);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	@Override
	protected void initModel() {
		List<User> users = CdmStore.getService(IUserService.class).list(User.class, null, null, null, null);
		
		model = new ArrayList<UuidAndTitleCache<User>>();
		
		for(User user : users){
			model.add(new UuidAndTitleCache<User>(User.class, user.getUuid(),String.format("%s, %s", user.getUsername(), user.getPerson())));
		}		 
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewWizardLinkText()
	 */
	@Override
	protected String getNewWizardLinkText() {
		return "Create a new <A>User</A>.";
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getNewEntityWizard(java.lang.String)
	 */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewUserWizard();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	@Override
	protected User getPersistentObject(UUID uuid) {
		return CdmStore.getService(IUserService.class).load(uuid);
	}

	@Override
	protected String getTitle(User user) {
		return user != null ? user.getUsername() : "";
	}
	
}
