package eu.etaxonomy.taxeditor.ui.element;


import java.util.List;
import java.util.Set;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * <p>ICdmFormElement interface.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public interface ICdmFormElement extends IPropertyChangeEmitter, IPropertyChangeListener {

	/**
	 * <p>getPropertyChangeListeners</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public abstract List<IPropertyChangeListener> getPropertyChangeListeners();

	/**
	 * <p>setPropertyChangeListeners</p>
	 *
	 * @param propertyChangeListeners a {@link java.util.Set} object.
	 */
	public abstract void setPropertyChangeListeners(
			List<IPropertyChangeListener> propertyChangeListeners);
	
	/**
	 * <p>getControls</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public abstract Set<Control> getControls();
	
	/**
	 * <p>getElements</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public abstract Set<ICdmFormElement> getElements();
	
	/**
	 * <p>addElement</p>
	 *
	 * @param element a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public abstract void addElement(ICdmFormElement element);
	
	/**
	 * <p>getParentElement</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public abstract ICdmFormElement getParentElement();
		
	/**
	 * <p>removeElements</p>
	 */
	public abstract void removeElements();
	
	/**
	 * <p>getLayoutComposite</p>
	 *
	 * @return a {@link org.eclipse.swt.widgets.Composite} object.
	 */
	public abstract Composite getLayoutComposite();
	
	/**
	 * <p>containsFormElement</p>
	 *
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @return a boolean.
	 */
	public boolean containsFormElement(ICdmFormElement formElement);
	
	/**
	 * <p>setBackground</p>
	 *
	 * @param color a {@link org.eclipse.swt.graphics.Color} object.
	 */
	public void setBackground(Color color);
	
	/**
	 * Sets a background color for this element and stores this background color
	 * for future reference
	 * 
	 * @param color
	 */
	public void setPersistentBackground(Color color);
	
	/**
	 * Get the background color that is associated with this element.
	 * 
	 * @return
	 */
	public Color getPersistentBackground();
	
	/**
	 * Refreshes the layout of this form element.
	 *
	 * This method should be called when the content of the form element
	 * changes and the current element needs to display that changes
	 */
	public void refresh();
}
