// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.common.CdmUtils;


/**
 * <p>NumberWithLabelElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 22, 2010
 * @version 1.0
 */
public class NumberWithLabelElement extends TextWithLabelElement {

	private Float start;
	private Float end;
	
	private NumberFormatException exception;

	/**
	 * <p>Constructor for NumberWithLabelElement.</p>
	 *
	 * @param toolkit a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param initialInteger a {@link java.lang.Integer} object.
	 * @param style a int.
	 */
	public NumberWithLabelElement(CdmFormFactory toolkit,
			ICdmFormElement parentElement, String labelString,
			Integer initialInteger, int style) {
		super(toolkit, parentElement, labelString, null, null, style);
		setInteger(initialInteger);
	}
	
	/**
	 * <p>Constructor for NumberWithLabelElement.</p>
	 *
	 * @param toolkit a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param initialFloat a {@link java.lang.Float} object.
	 * @param style a int.
	 */
	public NumberWithLabelElement(CdmFormFactory toolkit,
			ICdmFormElement parentElement, String labelString,
			Float initialFloat, int style) {
		super(toolkit, parentElement, labelString, null, null, style);
		setFloat(initialFloat);
	}
	
	/**
	 * <p>setInteger</p>
	 *
	 * @param number a {@link java.lang.Integer} object.
	 */
	public void setInteger(Integer number) {
		super.setText(getStringRepresentation(number));
	}
	
	/**
	 * <p>setFloat</p>
	 *
	 * @param number a {@link java.lang.Float} object.
	 */
	public void setFloat(Float number) {
		super.setText(getStringRepresentation(number));
	}
	
	/**
	 * <p>getInteger</p>
	 *
	 * @return a {@link java.lang.Integer} object.
	 */
	public Integer getInteger() {
		String text = super.getText().trim();
		return text.equals("") ? 0 : new Integer(text);
	}
	
	/**
	 * <p>getFloat</p>
	 *
	 * @return a {@link java.lang.Float} object.
	 */
	public Float getFloat(){
		String text = super.getText();
		return new Float(text);
	}
	
	private String getStringRepresentation(Object number){
		if(number != null){
			return number.toString();
		}		
		return null;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void modifyText(ModifyEvent event) {
		String value = text.getText();
		if(CdmUtils.isEmpty(value)){
			text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
			return;
		}
		
		try{
			
			Float number = Float.parseFloat(value);
			
			if((start != null && number < start) || (end != null && number > end)){
				throw new NumberFormatException("You entered a number that is not within the allowed bounds.");
			}
			
		}catch(NumberFormatException e){
			text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
			return;
		}
		
		exception = null;
		text.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		
		super.modifyText(event);
	}
	
	/**
	 * <p>setLimits</p>
	 *
	 * @param numberOfDigits a int.
	 * @param start a {@link java.lang.Integer} object.
	 * @param end a {@link java.lang.Integer} object.
	 */
	public void setLimits(int numberOfDigits, Integer start, Integer end){
		setLimits(numberOfDigits, start.floatValue(), end.floatValue());
	}
	
	/**
	 * <p>setLimits</p>
	 *
	 * @param numberOfDigits a int.
	 * @param start a {@link java.lang.Float} object.
	 * @param end a {@link java.lang.Float} object.
	 */
	public void setLimits(int numberOfDigits, Float start, Float end){
		text.setTextLimit(numberOfDigits);
		this.start = start;
		this.end = end;
	}
	
	/**
	 * <p>Getter for the field <code>exception</code>.</p>
	 *
	 * @return the exception
	 */
	public NumberFormatException getException() {
		return exception;
	}
}
