// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.element;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;

import eu.etaxonomy.taxeditor.store.StoreUtil;


/**
 * <p>SelectionArbitrator class.</p>
 *
 * @author n.hoffmann
 * @created Feb 25, 2010
 * @version 1.0
 */
public class SelectionArbitrator implements SelectionListener, ISelectionChangedListener{
	private IEntityElement entityElement;

	private Set<ISelectionProvider> selectionProviders = new HashSet<ISelectionProvider>();
	
	/**
	 * <p>Constructor for SelectionArbitrator.</p>
	 */
	public SelectionArbitrator(){}
	
	/**
	 * <p>Constructor for SelectionArbitrator.</p>
	 *
	 * @param selectableComposite a {@link eu.etaxonomy.taxeditor.ui.element.IEntityElement} object.
	 */
	public SelectionArbitrator(IEntityElement selectableComposite){
		this.entityElement = selectableComposite;
	}
	
	/** {@inheritDoc} */
	public void widgetSelected(SelectionEvent e) {
		Widget widget = e.widget;
		
		if(widget instanceof Control){
			Control control = (Control) widget;
			if(isWidgetPartOfFormElement(control, entityElement)){
				if(entityElement.getEntity() != null){
					IStructuredSelection selection = new StructuredSelection(entityElement.getEntity());
					setSelection(selection);
				}
			}
		}	
	}
	
	private void setSelection(ISelection selection){
		for(ISelectionProvider selectionProvider : selectionProviders){
			selectionProvider.setSelection(selection);
		}
	}
	
	private boolean isWidgetPartOfFormElement(Control widget, ICdmFormElement element){
		// check if widget is the controls
		if(widget.equals(element.getLayoutComposite())){
			return true;
		}
		
		// check if widget is controlled by the given element
		for(Control control : element.getControls()){
			if(control.equals(widget)) return true;
		}
		
		// descend into child elements
		for(ICdmFormElement childElement : element.getElements()){
			isWidgetPartOfFormElement(widget, childElement);
		}
		
		return false;
	}
	
	/**
	 * <p>addSelectionProvider</p>
	 *
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 */
	public void addSelectionProvider(ISelectionProvider selectionProvider){
		selectionProviders.add(selectionProvider);
	}
	
	/**
	 * <p>removeSelectionProvider</p>
	 *
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 */
	public void removeSelectionProvider(ISelectionProvider selectionProvider){
		selectionProviders.remove(selectionProvider);
	}
	
	/** {@inheritDoc} */
	public void widgetDefaultSelected(SelectionEvent e) {}

	/**
	 * <p>Getter for the field <code>entityElement</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.element.IEntityElement} object.
	 */
	public IEntityElement getEntityElement() {
		return entityElement;
	}

	/**
	 * <p>setEntityComposite</p>
	 *
	 * @param entityElement a {@link eu.etaxonomy.taxeditor.ui.element.IEntityElement} object.
	 */
	public void setEntityComposite(IEntityElement entityElement) {
		this.entityElement = entityElement;
	}

	/** {@inheritDoc} */
	public void selectionChanged(SelectionChangedEvent event) {
//		if(!(event.getSource() instanceof CdmDetailsViewer)){
//			return;
//		}
		
		try{
		entityElement.setSelected(false);
		}catch(SWTException e){
			StoreUtil.error(getClass(), "Widgets of element are disposed", e);
		}
		ISelection selection = event.getSelection();		
		if(selection != null && selection instanceof StructuredSelection){
			Object selectedElement = ((StructuredSelection) selection).getFirstElement();
			if(entityElement.getEntity() != null && entityElement.getEntity().equals(selectedElement)){
				entityElement.setSelected(true);
			}
		}
	}
}
