// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.password;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;

/**
 * @author n.hoffmann
 * @created Mar 10, 2011
 * @version 1.0
 */
public class PasswordWizard extends Wizard implements IConversationEnabled{

	private PasswordWizardPage passwordPage;
	private User user;
	private ConversationHolder conversation;
	
	public PasswordWizard(ConversationHolder conversation, User user){
		this.conversation = conversation;
		this.user = user;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		CdmFormFactory formFactory = new CdmFormFactory(Display.getDefault());
		
		passwordPage = new PasswordWizardPage(formFactory, getConversationHolder(), user);
		addPage(passwordPage);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		String userName = user.getUsername();
		ConversationHolder internalConversation = CdmStore.getLoginManager().getConversationHolder();
		internalConversation.bind();
		try{
			if(CdmStore.getLoginManager().isAdmin()){
				CdmStore.getService(IUserService.class).changePasswordForUser(userName, passwordPage.getNewPassword());
			}else{
				CdmStore.getService(IUserService.class).changePassword(passwordPage.getOldPassword(), passwordPage.getNewPassword());
			}
			internalConversation.commit();
			return true;
		}catch(Exception e){
			StoreUtil.errorDialog("Error while changing password", getClass(), "There was a problem changing the password.", e);
			return false;
		}finally{
			conversation.bind();
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	@Override
	public void update(CdmDataChangeMap arg0) {
		// TODO Auto-generated method stub
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	@Override
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

}
