// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * <p>
 * Abstract AbstractEntityCollectionElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 */
public abstract class AbstractEntityCollectionElement<ENTITY> extends
		AbstractCdmFormElement implements IEntityElement<ENTITY>,
		SelectionListener, IConversationEnabled, ISelectable {

	protected ENTITY entity;

	private Composite container;

	private Composite box;

	private Button button_remove;

	private Color backgroundColor;

	/**
	 * <p>
	 * Constructor for AbstractEntityCollectionElement.
	 * </p>
	 * 
	 * @param style
	 *            a int.
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param section
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *            object.
	 * @param entity
	 *            a ENTITY object.
	 * @param removeListener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param <ENTITY>
	 *            a ENTITY object.
	 */
	public AbstractEntityCollectionElement(CdmFormFactory formFactory,
			AbstractFormSection section, ENTITY entity,
			SelectionListener removeListener, Color backgroundColor, int style) {
		super(formFactory, (ICdmFormElement) section);

		init();

		formFactory.addPropertyChangeListener(this);

		// section.getLayoutComposite().setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_CYAN));

		box = formFactory.createComposite(section.getLayoutComposite());
		box.setBackgroundMode(SWT.INHERIT_DEFAULT);
		addControl(box);

		TableWrapLayout boxLayout = LayoutConstants.LAYOUT(2, false);
		boxLayout.topMargin = 4;
		boxLayout.bottomMargin = 4;
		box.setLayout(boxLayout);

		box.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		// box.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));

		container = formFactory.createComposite(box);
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);

		// container.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
		setLayoutComposite(container);

		addControl(container);
		Layout containerLayout = LayoutConstants.LAYOUT(2, false);

		container.setLayout(containerLayout);
		container.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		if (removeListener != null) {
			button_remove = formFactory.createButton(box, null, SWT.PUSH);
			addControl(button_remove);
			button_remove.setLayoutData(LayoutConstants.RIGHT());
			button_remove.setImage(ImageResources
					.getImage(ImageResources.TRASH_ICON));
			button_remove.setToolTipText("Remove");

			button_remove.addSelectionListener(removeListener);
		}

		createControls(this, style);

		setEntity(entity);
	}

	/**
	 * Init gets executed before any other setup of the section takes place
	 * 
	 * Implement this if you want to configure the section
	 */
	public void init() {
		// default implementation is empty
	}

	/**
	 * <p>
	 * Setter for the field <code>entity</code>.
	 * </p>
	 * 
	 * @param entity
	 *            a ENTITY object.
	 */
	public abstract void setEntity(ENTITY entity);

	/**
	 * <p>
	 * Getter for the field <code>entity</code>.
	 * </p>
	 * 
	 * @return a ENTITY object.
	 */
	public ENTITY getEntity() {
		return entity;
	}

	/**
	 * <p>
	 * createControls
	 * </p>
	 * 
	 * @param element
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param style
	 *            a int.
	 */
	public abstract void createControls(ICdmFormElement element, int style);

	/**
	 * Mark <code>this</code> element as selected.
	 */
	public void setSelected(boolean selected) {

		for (ICdmFormElement element : getElements()) {
			if (element instanceof ISelectable) {
				((ISelectable) element).setSelected(selected);
			}
		}
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.forms.AbstractCdmFormElement#propertyChange(org
	 * .eclipse.jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();
		if (getElements().contains(eventSource)) {
			handleEvent(eventSource);
		}
	}

	/**
	 * <p>
	 * handleEvent
	 * </p>
	 * 
	 * @param eventSource
	 *            a {@link java.lang.Object} object.
	 */
	public abstract void handleEvent(Object eventSource);

	/** {@inheritDoc} */
	public void setBackground(Color color) {
		backgroundColor = color;
		super.setBackground(color);
		box.setBackground(color);
		container.setBackground(color);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * React when selection occurs
	 */
	public void widgetSelected(SelectionEvent e) {

	}

	/** {@inheritDoc} */
	public void widgetDefaultSelected(SelectionEvent e) {
	}

	/** {@inheritDoc} */
	@Override
	public Composite getLayoutComposite() {
		return container;
	}

	/**
	 * <p>
	 * Getter for the field <code>backgroundColor</code>.
	 * </p>
	 * 
	 * @return the backgroundColor
	 */
	public Color getBackgroundColor() {
		return backgroundColor;
	}

	/**
	 * <p>
	 * getConversationHolder
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder}
	 *         object.
	 */
	public ConversationHolder getConversationHolder() {
		if (getParentElement() instanceof IConversationEnabled) {
			return ((IConversationEnabled) getParentElement())
					.getConversationHolder();
		}
		throw new IllegalArgumentException(
				"Parent element should be IConversationEnabled");
	}

	/** {@inheritDoc} */
	public void update(CdmDataChangeMap changeEvents) {
	}
}
