// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.classification;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.parser.ParseHandler;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.TaxonNodeSelectionElement;

/**
 * <p>
 * TaxonNodeDetailElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Sep 27, 2010
 * @version 1.0
 */
public class TaxonNodeDetailElement extends AbstractCdmDetailElement<ITreeNode> {

	private EntitySelectionElement<Classification> selection_classification;

	private Classification classification;

	private TaxonNodeSelectionElement selection_parentTaxonNode;

	private ITreeNode parentTreeNode;

	private EntitySelectionElement<Taxon> selection_reuseExistingTaxon;

	private Taxon taxon;

	private TextWithLabelElement text_newTaxonName;

	private CheckboxElement checkbox_openInEditor;

	private boolean openInEditor;

	private boolean complete;

	private EntitySelectionElement<TaxonNameBase> selection_reuseExistingName;

	/**
	 * <p>
	 * Constructor for TaxonNodeDetailElement.
	 * </p>
	 * 
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 */
	public TaxonNodeDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement,
	 * eu.etaxonomy.cdm.model.common.IAnnotatableEntity, int)
	 */
	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			ITreeNode entity, int style) {
		selection_classification = formFactory
				.createSelectionElement(Classification.class,
						getConversationHolder(), formElement, "Classification",
						null, EntitySelectionElement.DELETABLE, style);
		selection_parentTaxonNode = formFactory
				.createTaxonNodeSelectionElement(getConversationHolder(), formElement, "Parent", null,
						EntitySelectionElement.DELETABLE, style);
		selection_reuseExistingTaxon = formFactory
				.createSelectionElement(Taxon.class,
						getConversationHolder(), formElement,
						"Reuse existing Taxon", null,
						EntitySelectionElement.NOTHING, style);

		selection_reuseExistingName = formFactory
				.createSelectionElement(TaxonNameBase.class,
						getConversationHolder(), formElement,
						"Reuse existing name", null,
						EntitySelectionElement.ALL, style);

		text_newTaxonName = formFactory.createTextWithLabelElement(formElement,
				"New Taxon", "", style);

		checkbox_openInEditor = formFactory.createCheckbox(formElement,
				"Open in Editor", true, style);
		setOpenInEditor(true);
		setParentTreeNode(entity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
	 * .lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == selection_classification) {
			setClassification(selection_classification.getEntity());
		} else if (eventSource == selection_parentTaxonNode) {
			setParentTreeNode(selection_parentTaxonNode.getEntity());
		} else if (eventSource == selection_reuseExistingTaxon) {
			boolean enabled = selection_reuseExistingTaxon.getEntity() == null;
			selection_reuseExistingName.setEnabled(enabled);
			text_newTaxonName.setEnabled(enabled);

			setTaxon(selection_reuseExistingTaxon.getEntity());
		} else if (eventSource == selection_reuseExistingName) {
			boolean enabled = selection_reuseExistingName.getEntity() == null;
			selection_reuseExistingTaxon.setEnabled(enabled);
			text_newTaxonName.setEnabled(enabled);

			setTaxon(selection_reuseExistingName.getEntity());
		} else if (eventSource == text_newTaxonName) {
			boolean enabled = CdmUtils.isEmpty(text_newTaxonName.getText());
			selection_reuseExistingTaxon.setEnabled(enabled);
			selection_reuseExistingName.setEnabled(enabled);

			setTaxon(text_newTaxonName.getText());
		} else if (eventSource == checkbox_openInEditor) {
			setOpenInEditor(checkbox_openInEditor.getSelection());
		}
	}

	/**
	 * <p>
	 * Getter for the field <code>classification</code>.
	 * </p>
	 * 
	 * @return the classification
	 */
	public Classification getClassification() {
		return classification;
	}

	/**
	 * <p>
	 * isOpenInEditor
	 * </p>
	 * 
	 * @return the openInEditor
	 */
	public boolean isOpenInEditor() {
		return openInEditor;
	}

	/**
	 * <p>
	 * Getter for the field <code>parentTreeNode</code>.
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.cdm.model.taxon.ITreeNode} object.
	 */
	public ITreeNode getParentTreeNode() {
		return parentTreeNode;
	}

	/**
	 * <p>
	 * Getter for the field <code>taxon</code>.
	 * </p>
	 * 
	 * @return the taxon
	 */
	public Taxon getTaxon() {
		return taxon;
	}

	/**
	 * @param classification
	 *            the classification to set
	 */
	private void setClassification(Classification classification) {
		this.classification = classification;
		setParentTreeNode(classification);
	}

	/**
	 * @param parentTreeNode
	 *            the parentTreeNode to set
	 */
	private void setParentTreeNode(ITreeNode parentTreeNode) {
		this.parentTreeNode = parentTreeNode;

		if (parentTreeNode instanceof Classification) {
			classification = (Classification) parentTreeNode;
			selection_classification.setEntity(classification);
			selection_parentTaxonNode.setEntity(null);
			selection_parentTaxonNode.setClassification(classification);
		} else if (parentTreeNode instanceof TaxonNode) {
			classification = (Classification) HibernateProxyHelper
					.deproxy(((TaxonNode) parentTreeNode).getClassification());
			selection_classification.setEntity(classification);
			selection_parentTaxonNode.setEntity((TaxonNode) parentTreeNode);
			selection_parentTaxonNode.setClassification(classification);
		} else if(parentTreeNode == null){
			this.parentTreeNode = selection_classification.getEntity();			
		}
	}

	/**
	 * @param reuseExistingTaxon
	 *            the reuseExistingTaxon to set
	 */
	private void setTaxon(Taxon taxon) {
		this.taxon = taxon;
	}

	private void setTaxon(String taxonNameString) {
		TaxonNameBase taxonName = ParseHandler.parseReferencedName(
				taxonNameString, null);
		setTaxon(taxonName);
	}

	private void setTaxon(TaxonNameBase taxonName) {
		Reference secundum = null;
		if (getParentTreeNode() != null) {
			if (getParentTreeNode() instanceof Classification) {
				secundum = ((Classification) getParentTreeNode())
						.getReference();
			} else if (getParentTreeNode() instanceof TaxonNode) {
				secundum = ((TaxonNode) getParentTreeNode()).getTaxon()
						.getSec();
			}
		}
		taxon = Taxon.NewInstance(taxonName, secundum);
	}

	/**
	 * @param openInEditor
	 *            the openInEditor to set
	 */
	private void setOpenInEditor(boolean openInEditor) {
		this.openInEditor = openInEditor;
	}

	/**
	 * <p>
	 * isComplete
	 * </p>
	 * 
	 * @return the complete
	 */
	public boolean isComplete() {
		return complete;
	}

}
