// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import org.eclipse.swt.SWT;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>DescriptionDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 5, 2010
 * @version 1.0
 */
public class DescriptionDetailElement extends AbstractCdmDetailElement<DescriptionBase> {

	private TextWithLabelElement detailDescription;
	private EntitySelectionElement<FeatureTree> selection_featureTree;

	/**
	 * <p>Constructor for DescriptionDetailElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public DescriptionDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}
	
	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement, DescriptionBase entity, int style) {
		detailDescription = formFactory.createMultilineTextWithLabel(this, "Label", 50, SWT.WRAP);
		detailDescription.setText(entity.getTitleCache());
		
		// TODO I initially thought that we can handle the entity's descriptive system with a feature tree 
		// Since descriptiveSystem is a Set of Features that are present or allowed for the current description, 
		// it is similar to a FeatureTree (you can get this information from a feature tree) but not the same.
		// Please correct this once it is clear what descriptive system will be used for in CDM
//		selection_featureTree = formFactory.createSelectionElement(FeatureTree.class, getConversationHolder(), formElement, "Feature Tree", null, EntitySelectionElement.ALL, style);
	}
		
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == detailDescription){
			getEntity().setTitleCache(detailDescription.getText(), true);
		}
		else if(eventSource == selection_featureTree){
			FeatureTree featureTree = selection_featureTree.getSelection();
			getEntity().setDescriptiveSystem(featureTree.getDistinctFeatures());
		}
	}
}
