// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>StateDataSection class.</p>
 *
 * @author n.hoffmann
 * @created Sep 15, 2010
 * @version 1.0
 */
public class StateDataSection extends AbstractEntityCollectionSection<CategoricalData, StateData> {

	/**
	 * <p>Constructor for StateDataSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public StateDataSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(formFactory, conversation, parentElement, "State Data", style);
	}
	
	/** {@inheritDoc} */
	@Override
	public Collection<StateData> getCollection(CategoricalData entity) {
		return entity.getStates();
	}

	/** {@inheritDoc} */
	@Override
	public StateData createNewElement() {
		return StateData.NewInstance();
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(StateData element) {
		getEntity().addState(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(StateData element) {
		getEntity().removeState(element);
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No state data yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create new state data";
	}

	
}
