// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.feature;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.common.UriUtils;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.BrowserElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * <p>FeatureDistributionDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Sep 23, 2010
 * @version 1.0
 */
public class FeatureDistributionDetailElement extends AbstractCdmDetailElement<FeatureNodeContainer> {
	
	private BrowserElement image;
	
	/**
	 * <p>Constructor for FeatureDistributionDetailElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public FeatureDistributionDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls(eu.etaxonomy.taxeditor.forms.ICdmFormElement, eu.etaxonomy.cdm.model.common.AnnotatableEntity, int)
	 */
	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement, FeatureNodeContainer entity,
			int style) {
		String serviceUriString = PreferencesUtil.getPreferenceStore().getString(IPreferenceKeys.EDIT_MAP_SERVICE_ACCES_POINT);
		String message = null;
		try {
			URI serviceUri = new URI(serviceUriString);
			if(UriUtils.isServiceAvailable(serviceUri)){
				image = formFactory.createBrowserElement(formElement, null, style);
				String mapUriString = getMapUriString(getEntity());
				image.setImageUriString(mapUriString);
				formElement.getLayoutComposite().layout();
				message = mapUriString; 
				return;
			} else {
				message = String.format("The service is not available: %s", serviceUriString);
			}
		} catch (URISyntaxException e) {
			message = String.format("The URI has problems: %s", serviceUriString);
		}
		
		Label label = formFactory.createLabel(getLayoutComposite(), message, SWT.WRAP);
		addControl(label);
		label.setLayoutData(LayoutConstants.FILL(2, 1));
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		// 
	}

	/**
	 * @return
	 * @throws URISyntaxException 
	 */
	private String getMapUriString(FeatureNodeContainer container){
		String accessPoint = PreferencesUtil.getMapServiceAccessPoint();
		Map<PresenceAbsenceTermBase<?>, java.awt.Color> presenceAbsenceTermColors = null;
		// FIXME due to a bug in the rest map service we have to ensure that width will always be an even number
		// image.calculateWidth() % 2 == 1 ? image.calculateWidth() + 1 : 
		int width = image.calculateWidth();
		
		String bbox = "-180,-90,180,90"; 
		String backLayer = null;

		List<Language> languages = Arrays.asList(new Language[]{CdmStore.getDefaultLanguage()});
		
		String parameter = CdmStore.getGeoService().getDistributionServiceRequestParameterString(getTaxonDescriptions(), presenceAbsenceTermColors, width, 0, bbox, backLayer, languages);
		
		String mapUriString = String.format("%s?%s&ms=1000", accessPoint, parameter);
		return mapUriString;
	}
	
	/**
	 * @return
	 */
	private List<TaxonDescription> getTaxonDescriptions() {
		return Arrays.asList(((TaxonDescription) getEntity().getDescription()).getTaxon().getDescriptions().toArray(new TaxonDescription[0]));
	}

}
