// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.media;

import java.io.IOException;
import java.net.URI;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpException;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.common.media.ImageInfo;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.ImageElement;
import eu.etaxonomy.taxeditor.ui.element.KeyValueViewerElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;

/**
 * <p>ImageFileElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 26, 2010
 * @version 1.0
 */
public class ImageFileElement extends MediaRepresentationPartElement<ImageFile> {

	private NumberWithLabelElement text_height;
	private NumberWithLabelElement text_width;
	private KeyValueViewerElement element_keyValue;
	private ImageElement element_image;
	
	
	private Runnable postRunnable = new Runnable(){
		public void run() {
			StoreUtil.reflowDetailsViewer();
			StoreUtil.reflowSupplementalViewer();
		}
	};
	
	
	/**
	 * <p>Constructor for ImageFileElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param section a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param element a {@link eu.etaxonomy.cdm.model.media.ImageFile} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 */
	public ImageFileElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection section, ImageFile element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, section, element, removeListener, style);
	}
	
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement formElement, int style) {
		super.createControls(formElement, style);
		
		text_height = formFactory.createIntegerTextWithLabelElement(formElement, "Height", null, style);
		text_height.setEnabled(false);
		text_width = formFactory.createIntegerTextWithLabelElement(formElement, "Width", null, style);
		text_width.setEnabled(false);
		element_image = formFactory.createImageElement(formElement, null, style);
		element_keyValue = formFactory.createKeyValueViewerElement(formElement, "Key", "Value", null);
	}
	
	/** {@inheritDoc} */
	@Override
	public void setEntity(ImageFile entity) {
		super.setEntity(entity);
		text_height.setInteger(entity.getHeight());
		text_width.setInteger(entity.getWidth());
		try {
			loadImage(entity.getUri(), false);		
		} catch (Exception e) {
			handleException(e);
		}
	}
	
	private void loadImage(URI uri, boolean updateDimensions) throws IOException, HttpException{
		element_image.initImageUri(uri);
		
		element_image.loadImage(postRunnable );
		
		
		if(uri == null){
			return;
		}
		
		ImageInfo imageInfo = ImageInfo.NewInstanceWithMetaData(uri, 10000);
				
		element_keyValue.setInput(imageInfo.getMetaData());
		
		if(updateDimensions){
			text_size.setText(FileUtils.byteCountToDisplaySize(imageInfo.getLength()));
			// KLUDGE this is not save for very large files, because of the int cast. 
			// But then, I don't think we will handle such large files in the near future
			getEntity().setSize((int) imageInfo.getLength());
			
			text_height.setInteger(imageInfo.getHeight());
			getEntity().setHeight(imageInfo.getHeight());
				
			text_width.setInteger(imageInfo.getWidth());
			getEntity().setWidth(imageInfo.getWidth());
		}
		
		
		element_messages.setText("");
	}

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_uri){
			try {
				String uriString = text_uri.getText();
				URI uri = StringUtils.isNotEmpty(uriString) ? new URI(uriString) : null;
				getEntity().setUri(uri);
				
				
				loadImage(getEntity().getUri(), true);
				
				firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
			} catch (Exception e) {
				handleException(e);
			} finally {				
				firePropertyChangeEvent(this);
			}
		}
	}

	protected void handleException(Exception e) {
		super.handleException(e);
		element_image.unloadImage(postRunnable);
		text_height.setInteger(0);
		text_width.setInteger(0);
		
		element_image.loadImage(new Runnable(){
			public void run() {
				StoreUtil.reflowDetailsViewer();
				StoreUtil.reflowSupplementalViewer();
			}
		});
	}
}
