/**
 * 
 */
package eu.etaxonomy.taxeditor.ui.section.media;

import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>
 * MediaElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class MediaElement extends AbstractEntityCollectionElement<Media> {

	private MediaRepresentationSection section_mediaRepresentation;
	private EntitySelectionElement<AgentBase> selection_artist;
	private LanguageStringWithLabelElement text_description;
	private LanguageStringWithLabelElement text_title;

	/**
	 * <p>
	 * Constructor for MediaElement.
	 * </p>
	 * 
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection}
	 *            object.
	 * @param element
	 *            a {@link eu.etaxonomy.cdm.model.media.Media} object.
	 * @param removeListener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style
	 *            a int.
	 * @param cdmFormFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 */
	public MediaElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection formElement, Media element,
			SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, element, removeListener, null, style);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite
	 * #createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_title = formFactory.createLanguageStringWithLabelElement(element,
				"Title", null, style);
		selection_artist = formFactory
				.createSelectionElement(AgentBase.class,
						getConversationHolder(), element, "Artist", null,
						EntitySelectionElement.ALL, style);
		text_description = formFactory.createLanguageStringWithLabelElement(
				element, "Description", null, 100, style);
		section_mediaRepresentation = (MediaRepresentationSection) formFactory
				.createEntityDetailSection(
						EntityDetailType.MEDIAREPRESENTATION,
						getConversationHolder(), element, style);
		section_mediaRepresentation.setLayoutData(LayoutConstants
				.FILL_HORIZONTALLY(2, 1));
	}

	/** {@inheritDoc} */
	@Override
	public void setEntity(Media element) {
		this.entity = element;
		section_mediaRepresentation.setEntity(element);
		selection_artist.setEntity((TeamOrPersonBase) HibernateProxyHelper
				.deproxy(element.getArtist()));

		text_description.setLanguageString(element.getDescription(CdmStore
				.getDefaultLanguage()));

		LanguageString title = element.getTitle(CdmStore.getDefaultLanguage());

		if (title == null) {
			title = LanguageString.NewInstance("",
					CdmStore.getDefaultLanguage());
		}

		text_title.setLanguageString(title);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractEntityCollectionElement#handleEvent
	 * (java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == selection_artist) {
			getEntity().setArtist(selection_artist.getSelection());
		} else if (eventSource == text_description) {
			getEntity().addDescription(text_description.getLanguageString());
		} else if (eventSource == text_title) {
			getEntity().putTitle(text_title.getLanguageString());
			if (getParentElement() instanceof AbstractCdmDetailSection) {
				((AbstractCdmDetailSection) getParentElement()).updateTitle();
			}
		}
		firePropertyChangeEvent(this);
	}
}
