// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.media;

import java.net.URI;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionElement;

/**
 * <p>Abstract AbstractMediaRepresentationPartElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 24, 2010
 * @version 1.0
 */
public class MediaRepresentationPartElement<T extends MediaRepresentationPart> extends
		AbstractEntityCollectionElement<T> {

	protected TextWithLabelElement text_uri;
	protected NumberWithLabelElement text_size;

	protected LabelElement element_messages;
	
	/**
	 * <p>Constructor for AbstractMediaRepresentationPartElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param section a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param element a T object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 * @param <T> a T object.
	 */
	public MediaRepresentationPartElement(CdmFormFactory formFactory,
			AbstractFormSection section,
			T element, SelectionListener removeListener,
			int style) {
		super(formFactory, section, element, removeListener, null, style);
	}
	
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		text_uri = formFactory.createTextWithLabelElement(element, "URI", null, style);
		text_size = formFactory.createIntegerTextWithLabelElement(element, "Size", null, style);
		text_size.setEnabled(false);
		
		element_messages = formFactory.createLabel(element, "");
	}

	@Override
	public void setEntity(T entity) {
		this.entity = entity;
		if(entity.getSize() != null){
			text_size.setText(FileUtils.byteCountToDisplaySize(entity.getSize()));
		}
		if(entity.getUri() != null){
			text_uri.setText(entity.getUri().toString());
		}
		element_messages.setText("We currrently do not know how to display this generic MediaRepresentationPart to you.");
	}

	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_uri){
			try {
				String uriString = text_uri.getText();
				URI uri = StringUtils.isNotEmpty(uriString) ? new URI(uriString) : null;
				getEntity().setUri(uri);
			} catch (Exception e) {
				handleException(e);
			} finally {				
				firePropertyChangeEvent(this);
			}
		} 
	}
	
	protected void handleException(Exception e) {
		element_messages.setText(e.getMessage());
		text_size.setText("");
	}

}
