// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.Arrays;

import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.taxeditor.model.AuthorHelper;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractIdentifiableEntityDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;
import eu.etaxonomy.taxeditor.ui.selection.NomenclaturalAuthorTeamSelectionElement;

/**
 * <p>
 * AuthorshipDetailElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Mar 4, 2010
 * @version 1.0
 */
public class AuthorshipDetailElement extends
	AbstractIdentifiableEntityDetailElement<NonViralName> implements
		IEnableableFormElement {

	private NomenclaturalAuthorTeamSelectionElement selection_exBasionymAuthorTeam;
	private NomenclaturalAuthorTeamSelectionElement selection_basionymAuthorTeam;
	private NomenclaturalAuthorTeamSelectionElement selection_exCombinationAuthorTeam;
	private NomenclaturalAuthorTeamSelectionElement selection_combinationAuthorTeam;

	/**
	 * <p>
	 * Constructor for AuthorshipDetailElement.
	 * </p>
	 * 
	 * @param cdmFormFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param formElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param style
	 *            a int.
	 */
	public AuthorshipDetailElement(CdmFormFactory cdmFormFactory,
			ICdmFormElement formElement, int style) {
		super(cdmFormFactory, formElement);
	}

	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			NonViralName entity, int style) {
		toggleable_cache = formFactory.createToggleableTextField(this,
				"Authorship Cache", entity.getAuthorshipCache(),
				entity.isProtectedAuthorshipCache(), style);
		selection_combinationAuthorTeam = formFactory.createNomenclaturalAuthorTeamSelectionElement(
				getConversationHolder(),
				formElement, "Author",
				AuthorHelper.getAuthor(entity.getCombinationAuthorTeam()),
				EntitySelectionElement.ALL, style);
		addElement(selection_combinationAuthorTeam);
		selection_exCombinationAuthorTeam = formFactory.createNomenclaturalAuthorTeamSelectionElement(
				getConversationHolder(),
				formElement, "Ex Author",
				AuthorHelper.getAuthor(entity.getExCombinationAuthorTeam()),
				EntitySelectionElement.ALL, style);
		addElement(selection_exCombinationAuthorTeam);
		selection_basionymAuthorTeam = formFactory.createNomenclaturalAuthorTeamSelectionElement(
				getConversationHolder(),
				formElement, "Basionym Author",
				AuthorHelper.getAuthor(entity.getBasionymAuthorTeam()),
				EntitySelectionElement.ALL, style);
		addElement(selection_basionymAuthorTeam);
		selection_exBasionymAuthorTeam = formFactory.createNomenclaturalAuthorTeamSelectionElement(
				getConversationHolder(),
				formElement, "Ex Basionym Author",
				AuthorHelper.getAuthor(entity.getExBasionymAuthorTeam()),
				EntitySelectionElement.ALL, style);
		addElement(selection_exBasionymAuthorTeam);
	}

	/** {@inheritDoc} */
	@Override
	public void updateContent() {
		if (getEntity() == null) {
			setEntity(NonViralName.NewInstance(null));
		}
		
		super.updateContent();
		toggleable_cache.setEnabled(getEntity().isProtectedAuthorshipCache());

		if (this.isIrrelevant()) {
			setIrrelevant(isIrrelevant());
		} else {
			setIrrelevant(toggleable_cache.getState(),
					Arrays.asList(new Object[] { toggleable_cache }));
		}
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleable_cache) {
			getEntity().setAuthorshipCache(toggleable_cache.getText(),
					toggleable_cache.getState());
			if (!isIrrelevant())
				setIrrelevant(toggleable_cache.getState(),
						Arrays.asList(new Object[] { toggleable_cache }));
		} else if (eventSource == selection_combinationAuthorTeam) {
			getEntity().setCombinationAuthorTeam(
					(INomenclaturalAuthor) selection_combinationAuthorTeam
							.getSelection());
		} else if (eventSource == selection_exCombinationAuthorTeam) {
			getEntity().setExCombinationAuthorTeam(
					(INomenclaturalAuthor) selection_exCombinationAuthorTeam
							.getSelection());
		} else if (eventSource == selection_basionymAuthorTeam) {
			getEntity().setBasionymAuthorTeam(
					(INomenclaturalAuthor) selection_basionymAuthorTeam
							.getSelection());
		} else if (eventSource == selection_exBasionymAuthorTeam) {
			getEntity().setExBasionymAuthorTeam(
					(INomenclaturalAuthor) selection_exBasionymAuthorTeam
							.getSelection());
		}

//		if (eventSource != toggleable_cache) {
//			toggleable_cache.setText(getEntity().getAuthorshipCache());
//		}

//		 we have to notify the parent if this is embedded in the nonviral name
//		 section
//		 maybe we can handle this a little bit more elegant
//		if (getParentElement() instanceof AbstractCdmDetailSection)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(
					getParentElement(), null));
	}
	
	@Override
	public void updateToggleableCacheField() {
		if(! getEntity().isProtectedAuthorshipCache()){
			toggleable_cache.setText(getEntity().getAuthorshipCache());
		}
	}
}
