// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * <p>ProtologueSection class.</p>
 *
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class ProtologueSection extends AbstractEntityCollectionSection<TaxonNameBase, DescriptionElementBase> implements ITaxonBaseDetailSection{
	
	private TaxonBase taxonBase;

	/**
	 * <p>Constructor for ProtologueSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public ProtologueSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Protologues", style);
	}

	/** {@inheritDoc} */
	@Override
	public DescriptionElementBase createNewElement() {
		return TextData.NewInstance(Feature.PROTOLOGUE());
	}


	/** {@inheritDoc} */
	@Override
	public Collection<DescriptionElementBase> getCollection(TaxonNameBase entity) {
		Collection<DescriptionElementBase> result = new ArrayList<DescriptionElementBase>();
		List<TaxonNameDescription> descriptions = getTaxonNameDescriptions(entity);
		for (TaxonNameDescription desc : descriptions){
			result.addAll(desc.getElements());
		}
		return result;
//OLD:	return getTaxonNameDescription(entity).getElements();
	}


	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No protologues yet.";
	}


	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new protologue";
	}

	/** {@inheritDoc} */
	@Override
	public void addElement(DescriptionElementBase element) {
		//for simplification we always use the first description to add elements here
		getTaxonNameDescription(getEntity()).addElement(element);
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(DescriptionElementBase element) {
		List<TaxonNameDescription> descriptions = getTaxonNameDescriptions(getEntity());
		for (TaxonNameDescription desc : descriptions){
			desc.removeElement(element);
		} 
	}

	/** {@inheritDoc} */
	public void setTaxonBase(TaxonBase entity) {
		this.taxonBase = entity;
		NonViralName name = (NonViralName) HibernateProxyHelper.deproxy(entity.getName());
		setEntity(name);
	}
	
	/**
	 * NOTE: returns first description
	 * @param name
	 * @return
	 */
	private TaxonNameDescription getTaxonNameDescription(TaxonNameBase name){
		if(name.getDescriptions().size() == 0){
			name.addDescription(TaxonNameDescription.NewInstance(name));
		}
		return (TaxonNameDescription) name.getDescriptions().iterator().next();
	}
	
	/**
	 * Try to fix FIXME in {@link #getTaxonNameDescription(TaxonNameBase)}
	 * @param name
	 * @return
	 */
	private List<TaxonNameDescription> getTaxonNameDescriptions(TaxonNameBase<?,?> name){
		List<TaxonNameDescription> result = new ArrayList<TaxonNameDescription>();
		if(name.getDescriptions().size() == 0){
			name.addDescription(TaxonNameDescription.NewInstance(name));
		}
		for (TaxonNameDescription desc : name.getDescriptions()){
			result.add(desc);
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.section.ITaxonDetailSection#getTaxon(eu.etaxonomy.cdm.model.taxon.TaxonBase)
	 */
	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}
}
