// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.selection;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.Institution;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.taxeditor.model.TextHelper;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.section.agent.InstitutionWizardPage;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonWizardPage;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamWizardPage;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.CollectionWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldObservationWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventWizardPage;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GeneralWizardPage;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceWizardPage;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailWizardPage;

/**
 * <p>
 * EditFromSelectionWizard class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Jun 1, 2010
 * @version 1.0
 */
public class EditFromSelectionWizard extends Wizard implements
		SelectionListener {

	private final EntitySelectionElement selectionElement;

	/**
	 * <p>
	 * Constructor for EditFromSelectionWizard.
	 * </p>
	 * 
	 * @param selectionElement
	 *            a
	 *            {@link eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement}
	 *            object.
	 */
	public EditFromSelectionWizard(EntitySelectionElement selectionElement) {
		this.selectionElement = selectionElement;
		this.setWindowTitle(String.format("Edit %s", TextHelper.deproxyClassName(selectionElement.getEntity().getClass())));
	}

	/** {@inheritDoc} */
	@Override
	public void addPages() {

		CdmFormFactory formFactory = selectionElement.getFormFactory();
		ICdmBase entity = (ICdmBase) HibernateProxyHelper
				.deproxy(selectionElement.getEntity());

		if (entity instanceof Reference) {
			addPage(new ReferenceWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					(Reference) entity));
		} else if (entity instanceof Team) {
			addPage(new TeamWizardPage(formFactory,
					selectionElement.getConversationHolder(), (Team) entity));
		} else if (entity instanceof Person) {
			addPage(new PersonWizardPage(formFactory,
					selectionElement.getConversationHolder(), (Person) entity));
		} else if (entity instanceof NonViralName) {
			addPage(new NonViralNameWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					(NonViralName) entity));
		} else if (entity instanceof DerivedUnitBase) {

			DerivedUnitFacade facade;
			try {
				facade = DerivedUnitFacade.NewInstance(
						(DerivedUnitBase) entity,
						PreferencesUtil.getDerivedUnitConfigurator());
			} catch (DerivedUnitFacadeNotSupportedException e) {
				// we should never get here
				throw new IllegalStateException();
			}

			addPage(new GeneralWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			addPage(new GatheringEventWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			addPage(new FieldObservationWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
			addPage(new DerivedUnitBaseWizardPage(formFactory,
					selectionElement.getConversationHolder(), facade));
		} else if (entity instanceof Collection) {
			addPage(new CollectionWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					(Collection) entity));
		} else if (entity instanceof Institution) {
			addPage(new InstitutionWizardPage(formFactory,
					selectionElement.getConversationHolder(),
					(Institution) entity));
		} else if (entity instanceof User) {
			addPage(new UserDetailWizardPage(formFactory, 
					selectionElement.getConversationHolder(), 
					(User) entity));
		} else {
			StoreUtil.warningDialog("Missing interface", this,
					"No detail element for current selection");
		} 

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void widgetSelected(SelectionEvent e) {
		WizardDialog dialog = new WizardDialog(selectionElement.getShell(),
				this);
		if (dialog.open() == IStatus.OK) {
			selectionElement.updateFromWizard();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void widgetDefaultSelected(SelectionEvent e) {
	}
}
