// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.datasource;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.CdmMetaData.MetaDataPropertyName;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>CdmDataSourceLabelProvider class.</p>
 *
 * @author n.hoffmann
 * @created 14.04.2009
 * @version 1.0
 */
public class CdmDataSourceLabelProvider extends LabelProvider implements ITableLabelProvider {
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
	 */
	/** {@inheritDoc} */
	public Image getColumnImage(Object element, int columnIndex) {
		if(columnIndex == 0){
			ICdmDataSource dataSource = ((CdmMetaDataAwareDataSourceContainer) element).getDataSource();
			
			String key = isCurrentDataSource(dataSource) ? ImageResources.IMG_DATASOURCE_CONNECTED : ImageResources.IMG_DATASOURCE_DISCONNECTED;
			
			return ImageResources.getImage(key);
		}
		
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
	 */
	/** {@inheritDoc} */
	public String getColumnText(Object element, int columnIndex) {
		CdmMetaDataAwareDataSourceContainer container = (CdmMetaDataAwareDataSourceContainer) element;
				
		ICdmDataSource dataSource = container.getDataSource();
				
		switch (columnIndex){
			case 1:
				return dataSource.getName();
			case 2:
				if(dataSource.getServer() != null){
					return dataSource.getServer();
				}
				// h2 databases do not have server entries
				return "local";
			case 3:
				return container.isRunning() ? "Yes" : "No";
			case 4:
				return dataSource.getDatabaseType().getName();
			case 5:
				return dataSource.getDatabase();
			case 6:
				if (dataSource.getNomenclaturalCode() == null) {
					return "";
				}
				return dataSource.getNomenclaturalCode().name();
			case 7:
				return container.getMetaData(MetaDataPropertyName.DB_CREATE_DATE);
			case 8:
				return container.getMetaData(MetaDataPropertyName.DB_SCHEMA_VERSION);
			case 9:
				if(container.getMetaDataMap().get(MetaDataPropertyName.DB_SCHEMA_VERSION) != null){
					return container.isDataSourceCompatible() ? "Yes" : "No";
				}
				break;
			case 10:
				return container.getMetaData(MetaDataPropertyName.DB_CREATE_NOTE);
		}
		return CdmMetaDataAwareDataSourceContainer.DEFAULT_ENTRY;
	}
	
	private boolean isCurrentDataSource(ICdmDataSource dataSource){
		ICdmDataSource currentDataSource = CdmStore.getDataSource();
		
		return currentDataSource != null && currentDataSource.getName().equals(dataSource.getName());
	}
}
