// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.detail;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.view.AbstractCdmEditorViewPart;
import eu.etaxonomy.taxeditor.view.AbstractCdmViewPart;

/**
 * <p>DetailsViewPart class.</p>
 *
 * @author n.hoffmann
 * @created Jun 10, 2010
 * @version 1.0
 */
public class DetailsViewPart extends AbstractCdmEditorViewPart implements IPartContentHasSupplementalData{
	
	/** Constant <code>ID="eu.etaxonomy.taxeditor.editor.forms.det"{trunked}</code> */
	public static String ID = "eu.etaxonomy.taxeditor.view.detail";

	private DetailsViewer viewer;

	/** {@inheritDoc} */
	@Override
	public void createViewer(Composite parent) {
		//selectionService.addPostSelectionListener(this);
		
		viewer = new DetailsViewer(parent, this);
		getSite().setSelectionProvider(viewer);
	}

	/** {@inheritDoc} */
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(StoreUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}
		
		if(part == this){
			return;
		}
		
		if(!(selection instanceof IStructuredSelection)){
			return;
		}
		
		IStructuredSelection structuredSelection = (IStructuredSelection) selection;
		
		if((part instanceof IEditorPart) || (part instanceof AbstractCdmViewPart)) {
			if(structuredSelection.size() != 1){
				showEmptyPage();
				return;
			}
			
			// do not show details for feature nodes TODO really? 
			if(structuredSelection.getFirstElement() instanceof FeatureNodeContainer){
				// do show the map for distributions
				Feature feature = ((FeatureNodeContainer) ((IStructuredSelection) selection).getFirstElement()).getFeature();
				if(!feature.equals(Feature.DISTRIBUTION())){
					showEmptyPage();
					return;
				}
			}
			
			showViewer(part, structuredSelection);
		}else{
			showEmptyPage();		
		}	
	}
	
	/** {@inheritDoc} */
	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		
		viewer.setInput(objectAffectedByOperation);
		
		return super.postOperation(objectAffectedByOperation);
	}
	
	
	/** {@inheritDoc} */
	@Override
	public Viewer getViewer() {
		return viewer;
	}

	
	/** {@inheritDoc} */
	@Override
	public void dispose() {
		super.dispose();
		selectionService.removePostSelectionListener(this);
	}

	/**
	 * <p>onComplete</p>
	 *
	 * @return a boolean.
	 */
	public boolean onComplete() {
		return true;
	}
}
