
// $Id$
/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.view.detail;

import java.util.Set;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.Group;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.description.PolytomousKeyNode;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.reference.Reference;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;

import eu.etaxonomy.taxeditor.editor.UsageTermCollection;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.PolytomousKeyRelationship;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.ui.element.RootElement;
import eu.etaxonomy.taxeditor.ui.section.agent.PersonDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamDetailSection;
import eu.etaxonomy.taxeditor.ui.section.agent.TeamOrPersonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.common.ReferencedEntityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescribedSpecimenSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementDetailSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementMediaSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionElementSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.DescriptionSourceSection;
import eu.etaxonomy.taxeditor.ui.section.description.MediaDetailsSection;
import eu.etaxonomy.taxeditor.ui.section.description.NaturalLanguageSection;
import eu.etaxonomy.taxeditor.ui.section.description.ScopeSection;
import eu.etaxonomy.taxeditor.ui.section.feature.FeatureDistributionDetailSection;
import eu.etaxonomy.taxeditor.ui.section.grantedAuthority.GrantedAuthorityDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.GroupDetailSection;
import eu.etaxonomy.taxeditor.ui.section.group.MemberDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.GeographicalScopeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.PolytomousKeyNodeDetailSection;
import eu.etaxonomy.taxeditor.ui.section.key.ScopeRestrictionSection;
import eu.etaxonomy.taxeditor.ui.section.key.TaxonomicScopeSection;
import eu.etaxonomy.taxeditor.ui.section.name.NameRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.NomenclaturalStatusSection;
import eu.etaxonomy.taxeditor.ui.section.name.NonViralNameDetailSection;
import eu.etaxonomy.taxeditor.ui.section.name.ProtologueSection;
import eu.etaxonomy.taxeditor.ui.section.name.TypeDesignationSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DerivedUnitBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.DeterminationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.FieldObservationDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GatheringEventDetailSection;
import eu.etaxonomy.taxeditor.ui.section.occurrence.GeneralDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.NomenclaturalReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.reference.ReferenceDetailSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.ParsingMessagesSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonBaseDetailSection;
import eu.etaxonomy.taxeditor.ui.section.taxon.TaxonRelationshipDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.GroupsByUserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.user.UserDetailSection;
import eu.etaxonomy.taxeditor.ui.section.userecords.UseRecordDetailSection;
import eu.etaxonomy.taxeditor.ui.section.vocabulary.TermVocabularyDetailSection;
import eu.etaxonomy.taxeditor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.view.AbstractCdmViewPart;

/**
 * <p>
 * DetailsViewer class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Feb 12, 2010
 * @version 1.0
 */
public class DetailsViewer extends AbstractCdmDataViewer {

	private ISelection selection;

	/**
	 * <p>
	 * Constructor for DetailsViewer.
	 * </p>
	 * 
	 * @param parent
	 *            a {@link org.eclipse.swt.widgets.Composite} object.
	 * @param viewPart
	 *            a {@link eu.etaxonomy.taxeditor.view.AbstractCdmViewPart}
	 *            object.
	 */
	public DetailsViewer(Composite parent, AbstractCdmViewPart viewPart) {
		super(parent, viewPart);
	}

	// START HACK TO MAKE THE DERIVED UNIT FACADE WORK
	// since we are getting implementations of DerivedUnitBase from the bulk
	// editor
	// and not derived unit facade objects,

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.editor.view.AbstractCdmDataViewer#setInput(java
	 * .lang.Object)
	 */
	@Override
	public void setInput(Object input) {
		if (input instanceof DerivedUnitBase) {
			try {
				input = DerivedUnitFacade.NewInstance((DerivedUnitBase) input,
						PreferencesUtil.getDerivedUnitConfigurator());
			} catch (DerivedUnitFacadeNotSupportedException e) {
				StoreUtil.error(getClass(), e);
			}
		}
		super.setInput(input);
	}

	@Override
	protected void markViewPartDirty() {
		if (getInput() instanceof DerivedUnitFacade) {
			getViewPart().changed(
					((DerivedUnitFacade) getInput()).innerDerivedUnit());
		}

		super.markViewPartDirty();
	}

	// END HACK TO MAKE THE DERIVED UNIT FACADE WORK

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	/** {@inheritDoc} */
	@Override
	protected void showParts() {
	    // FIXME (CM) : Need to clean out this code.
	    // Too much type checking to decide which detail view to display.
	    // Need to build in a mechanism where navigators / editors are 'aware'
	    // of the corresponding detail viewer.
		if (getInput() instanceof TaxonBase) {
			createTaxonSections(rootElement);

		} else if (getInput() instanceof NonViralName) {
			createNameSections(rootElement);

		} else if (getInput() instanceof Reference) {
			createReferenceSections(rootElement);

		} else if (getInput() instanceof Team) {
			createTeamDetailSection(rootElement);

		} else if (getInput() instanceof Person) {
			createPersonDetailSection(rootElement);

		} else if (getInput() instanceof TeamOrPersonBase) {
			createTeamOrPersonBaseDetailSection(rootElement);

		} else if (getInput() instanceof DescriptionBase) {
			Set<Marker> descriptionMarkers = ((DescriptionBase) getInput()).getMarkers();
			MarkerType useMarkertype = (MarkerType) CdmStore.getService(ITermService.class).find(UsageTermCollection.uuidUseMarkerType);
			Boolean isUseDescription = false;
			for (Marker marker : descriptionMarkers) {
				if (marker.getMarkerType().equals(useMarkertype)) {
					isUseDescription = true;
				}
			}
			if (((DescriptionBase) getInput()).isImageGallery()) {
				createImageGallerySection(rootElement);

			} else if (isUseDescription) { 
				createUseDescriptionSection(rootElement);
			} else {
				createDescriptionSection(rootElement);

			}
		} else if (getInput() instanceof DescriptionElementBase) {
			Set<Marker> descriptionMarkers = ((DescriptionElementBase) getInput()).getInDescription().getMarkers();
			MarkerType useMarkertype = (MarkerType) CdmStore.getService(ITermService.class).find(UsageTermCollection.uuidUseMarkerType);
			Boolean isUseDescription = false;
			for (Marker marker : descriptionMarkers) {
				if (marker.getMarkerType().equals(useMarkertype)) {
					isUseDescription = true;
				}
			}
			
			if (isUseDescription == true && getInput() instanceof CategoricalData ) {
				createUseRecordSection(rootElement);
			}
			else {
				createDescriptionElementSection(rootElement);
			}

		} else if (getInput() instanceof Media) {
			createMediaElementSection(rootElement);

		} else if (getInput() instanceof DerivedUnitFacade) {
			createDerivedUnitBaseElementSection(rootElement);

		} else if (getInput() instanceof FeatureNodeContainer) {
			createFeatureDistributionSection(rootElement);

		} else if (getInput() instanceof PolytomousKey) {
			//createPolytomousKeySection(rootElement);

		} else if ((getInput() instanceof PolytomousKeyNode)
				|| (getInput() instanceof PolytomousKeyRelationship)) {
			createPolytomousKeyNodeSection(rootElement);

		} else if (getInput() instanceof User) {
			createUserSection(rootElement);

		} else if (getInput() instanceof Group) {
			createGroupSection(rootElement);

		} else if (getInput() instanceof TaxonRelationship) {
			createTaxonRelationshipSection(rootElement);

		} else if (getInput() instanceof TermVocabulary) {
			createTermVocabularySection(rootElement);

		} else if (getInput() instanceof DefinedTermBase) {
			createDefinedTermSection(rootElement);
		} else {
			destroySections();
		}
		layout();
	}

	
	/**
	 * @param rootElement
	 */
	private void createGroupSection(RootElement parent) {
		destroySections();

		GroupDetailSection groupDetailSection = (GroupDetailSection) formFactory
				.createCdmDetailSection(DetailType.GROUP,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);
		MemberDetailSection memberDetailSection = (MemberDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.MEMBER,
						getConversationHolder(), parent, Section.TWISTIE
								| Section.EXPANDED);
		GrantedAuthorityDetailSection grantedAuthorityDetailSection = (GrantedAuthorityDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.GRANTED_AUTHORITY,
						getConversationHolder(), parent, Section.TWISTIE
								| Section.EXPANDED);

		addPart(groupDetailSection);
		addPart(memberDetailSection);
		addPart(grantedAuthorityDetailSection);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#getSelection()
	 */
	/** {@inheritDoc} */
	@Override
	public ISelection getSelection() {
		return selection;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers
	 * .ISelection, boolean)
	 */
	/** {@inheritDoc} */
	@Override
	public void setSelection(ISelection selection, boolean reveal) {
		this.selection = selection;
		SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(
				this, selection);
		fireSelectionChanged(selectionChangedEvent);
	}

	/**
	 * createTaxonSections(RootElement parent)
	 * @param parent
	 */
	private void createTaxonSections(RootElement parent) {
		destroySections();

		TaxonBaseDetailSection taxonBaseDetailSection = (TaxonBaseDetailSection) formFactory
				.createCdmDetailSection(DetailType.TAXONBASE,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NonViralNameDetailSection nonViralNameSection = (NonViralNameDetailSection) formFactory
				.createCdmDetailSection(DetailType.NONVIRALNAME,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NomenclaturalReferenceDetailSection referenceDetailSection = (NomenclaturalReferenceDetailSection) formFactory
				.createCdmDetailSection(DetailType.NOMENCLATURALREFERENCE,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NomenclaturalStatusSection nomenclaturalStatusSection = (NomenclaturalStatusSection) formFactory
				.createEntityDetailSection(
						EntityDetailType.NOMENCLATURALSTATUS,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		ProtologueSection protologSection = (ProtologueSection) formFactory
				.createEntityDetailSection(EntityDetailType.PROTOLOG,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		TypeDesignationSection typeDesignationSection = (TypeDesignationSection) formFactory
				.createEntityDetailSection(EntityDetailType.TYPEDESIGNATION,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NameRelationshipDetailSection nameRelationshipSection = (NameRelationshipDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.NAME_RELATIONSHIP,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		ParsingMessagesSection parsingMessagesSection = (ParsingMessagesSection) formFactory
				.createCdmDetailSection(DetailType.PARSINGMESSAGE,
						getConversationHolder(), parent, this, Section.EXPANDED);

		addPart(taxonBaseDetailSection);
		addPart(nonViralNameSection);
		addPart(nomenclaturalStatusSection);
		addPart(protologSection);
		addPart(referenceDetailSection);
		addPart(typeDesignationSection);
		addPart(nameRelationshipSection);
		addPart(parsingMessagesSection);
	}

	/**
	 * createNameSections
	 * @param parent
	 */
	private void createNameSections(RootElement parent) {
		destroySections();
		NonViralNameDetailSection nonViralNameSection = (NonViralNameDetailSection) formFactory
				.createCdmDetailSection(DetailType.NONVIRALNAME,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NomenclaturalReferenceDetailSection referenceDetailSection = (NomenclaturalReferenceDetailSection) formFactory
				.createCdmDetailSection(DetailType.NOMENCLATURALREFERENCE,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NomenclaturalStatusSection nomenclaturalStatusSection = (NomenclaturalStatusSection) formFactory
				.createEntityDetailSection(
						EntityDetailType.NOMENCLATURALSTATUS,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		ProtologueSection protologSection = (ProtologueSection) formFactory
				.createEntityDetailSection(EntityDetailType.PROTOLOG,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		TypeDesignationSection typeDesignationSection = (TypeDesignationSection) formFactory
				.createEntityDetailSection(EntityDetailType.TYPEDESIGNATION,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NameRelationshipDetailSection nameRelationshipSection = (NameRelationshipDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.NAME_RELATIONSHIP,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		addPart(nonViralNameSection);
		addPart(nomenclaturalStatusSection);
		addPart(protologSection);
		addPart(referenceDetailSection);
		addPart(typeDesignationSection);
		addPart(nameRelationshipSection);
	}

	/**
	 * createReferenceSections
	 * @param parent
	 */
	private void createReferenceSections(RootElement parent) {
		destroySections();

		ReferenceDetailSection referenceDetailSection = (ReferenceDetailSection) formFactory
				.createCdmDetailSection(DetailType.REFERENCEBASE,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(referenceDetailSection);
	}

	/**
	 * createTeamOrPersonBaseDetailSection
	 * @param parent
	 */
	private void createTeamOrPersonBaseDetailSection(RootElement parent) {
		destroySections();
		TeamOrPersonBaseDetailSection teamOrPersonBaseDetailSection = (TeamOrPersonBaseDetailSection) formFactory
				.createCdmDetailSection(DetailType.TEAMORPERSONBASE,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(teamOrPersonBaseDetailSection);
	}

	/**
	 * createTeamDetailSection
	 * @param parent
	 */
	private void createTeamDetailSection(RootElement parent) {
		destroySections();
		TeamDetailSection teamDetailSection = (TeamDetailSection) formFactory
				.createCdmDetailSection(DetailType.TEAM,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(teamDetailSection);
	}
	
	/**
	 * createPersonDetailSection
	 * @param parent
	 */
	private void createPersonDetailSection(RootElement parent) {
		destroySections();
		PersonDetailSection personDetailSection = (PersonDetailSection) formFactory
				.createCdmDetailSection(DetailType.PERSON,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);
		addPart(personDetailSection);
	}

	/**
	 * createDescriptionElementSection
	 * @param parent
	 */
	private void createDescriptionElementSection(RootElement parent) {
		destroySections();

		DescriptionElementDetailSection descriptionElementDetailSection = (DescriptionElementDetailSection) formFactory
				.createCdmDetailSection(DetailType.DESCRIPTIONELEMENT,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DescriptionElementSourceSection descriptionElementSourceSection = (DescriptionElementSourceSection) formFactory
				.createEntityDetailSection(
						EntityDetailType.DESCRIPTIONELEMENTSOURCE,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DescriptionElementMediaSection descriptionElementMediaSection = (DescriptionElementMediaSection) formFactory
				.createEntityDetailSection(
						EntityDetailType.DESCRIPTIONELEMENTMEDIA,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		addPart(descriptionElementDetailSection);
		addPart(descriptionElementSourceSection);
		addPart(descriptionElementMediaSection);
	}

	/**
	 * createDescriptionSection
	 * @param parent
	 */
	private void createDescriptionSection(RootElement parent) {
		destroySections();
		DescriptionDetailSection descriptionDetailSection = (DescriptionDetailSection) formFactory
				.createCdmDetailSection(DetailType.DESCRIPTION,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		NaturalLanguageSection naturalLanguageSection = (NaturalLanguageSection) formFactory
				.createCdmDetailSection(DetailType.NATURAL_LANGUAGE,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DescribedSpecimenSection describedSpecimenSection = (DescribedSpecimenSection) formFactory
				.createEntityDetailSection(EntityDetailType.DESCRIBED_SPECIMEN,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DescriptionSourceSection descriptionSourceSection = (DescriptionSourceSection) formFactory
				.createEntityDetailSection(EntityDetailType.DESCRIPTIONSOURCE,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		ScopeSection scopeSection = (ScopeSection) formFactory
				.createEntityDetailSection(EntityDetailType.SCOPE,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		addPart(descriptionDetailSection);
		addPart(naturalLanguageSection);
		addPart(describedSpecimenSection);
		addPart(descriptionSourceSection);
		addPart(scopeSection);
	}
	
	/**
	 * Creates the use Description section
	 * @param parent
	 */
	private void createUseDescriptionSection(RootElement parent) {
		destroySections();
		DescriptionDetailSection descriptionDetailSection = (DescriptionDetailSection) formFactory
				.createCdmDetailSection(DetailType.DESCRIPTION,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DescriptionSourceSection descriptionSourceSection = (DescriptionSourceSection) formFactory
				.createEntityDetailSection(EntityDetailType.DESCRIPTIONSOURCE,
						getConversationHolder(), parent, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		addPart(descriptionDetailSection);
		addPart(descriptionSourceSection);
		
	}


	/**
	 * @param rootElement
	 */
	private void createImageGallerySection(RootElement parent) {
		destroySections();
		DescriptionDetailSection descriptionDetailSection = (DescriptionDetailSection) formFactory
				.createCdmDetailSection(DetailType.DESCRIPTION,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(descriptionDetailSection);
	}
	
	/**
	 * createMediaElementSection
	 * @param parent
	 */

	private void createMediaElementSection(RootElement parent) {
		destroySections();
		MediaDetailsSection mediaDetailSection = (MediaDetailsSection) formFactory
				.createCdmDetailSection(DetailType.MEDIA,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(mediaDetailSection);
	}

	/**
	 * createDerivedUnitBaseElementSection
	 * @param parent
	 */
	private void createDerivedUnitBaseElementSection(RootElement parent) {
		destroySections();

		GeneralDetailSection generalDetailSection = (GeneralDetailSection) formFactory
				.createCdmDetailSection(DetailType.DERIVED_UNIT_GENERAL,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		GatheringEventDetailSection gatheringEventDetailSection = (GatheringEventDetailSection) formFactory
				.createCdmDetailSection(DetailType.GATHERING_EVENT,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		FieldObservationDetailSection fieldObservationDetailSection = (FieldObservationDetailSection) formFactory
				.createCdmDetailSection(DetailType.FIELD_OBSERVATION,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DerivedUnitBaseDetailSection derivedUnitBaseDetailSection = (DerivedUnitBaseDetailSection) formFactory
				.createCdmDetailSection(DetailType.DERIVED_UNIT,
						getConversationHolder(), parent, this, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		DeterminationDetailSection determinationDetailSection = (DeterminationDetailSection) formFactory
				.createCdmDetailSection(DetailType.DETERMINATION,
						getConversationHolder(), parent, this, Section.TWISTIE);

		addPart(generalDetailSection);
		addPart(gatheringEventDetailSection);
		addPart(fieldObservationDetailSection);
		addPart(derivedUnitBaseDetailSection);
		addPart(determinationDetailSection);

	}

	/**
	 * @param rootElement
	 */
	private void createFeatureDistributionSection(RootElement parent) {
		destroySections();

		FeatureDistributionDetailSection featureDistributionSection = (FeatureDistributionDetailSection) formFactory
				.createCdmDetailSection(DetailType.FEATURE_DISTRIBUTION,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		addPart(featureDistributionSection);
	}

	private void createPolytomousKeySection(RootElement parent) {
		destroySections();

		PolytomousKeyDetailSection polytomousKeyDetailSection = (PolytomousKeyDetailSection) formFactory
				.createCdmDetailSection(DetailType.POLYTOMOUS_KEY,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		GeographicalScopeDetailSection geographicalScopeDetailSection = (GeographicalScopeDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.GEOGRAPHICAL_SCOPE,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		ScopeRestrictionSection scopeRestrictionSection = (ScopeRestrictionSection) formFactory
				.createEntityDetailSection(EntityDetailType.SCOPE_RESTRICTION,
						getConversationHolder(), parent, Section.TWISTIE);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);

		TaxonomicScopeSection taxonomicScopeSection = (TaxonomicScopeSection) formFactory
				.createEntityDetailSection(EntityDetailType.TAXONOMIC_SCOPE,
						getConversationHolder(), parent, Section.TWISTIE);

		addPart(polytomousKeyDetailSection);
		addPart(geographicalScopeDetailSection);
		addPart(scopeRestrictionSection);
		addPart(taxonomicScopeSection);
	}

	/**
	 * createPolytomousKeyNodeSection
	 * @param parent
	 */
	private void createPolytomousKeyNodeSection(RootElement parent) {
		destroySections();

        PolytomousKeyDetailSection polytomousKeyDetailSection = (PolytomousKeyDetailSection) formFactory
                .createCdmDetailSection(DetailType.POLYTOMOUS_KEY,
                        getConversationHolder(), parent, this, Section.TWISTIE
                               | Section.COMPACT);
        
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
        
        PolytomousKeyNodeDetailSection polytomousKeyNodeDetailSection = (PolytomousKeyNodeDetailSection) formFactory
                .createCdmDetailSection(DetailType.POLYTOMOUS_KEY_NODE,
                        getConversationHolder(), parent, this, Section.TWISTIE
                                | Section.EXPANDED);

//        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//
//        GeographicalScopeDetailSection geographicalScopeDetailSection = (GeographicalScopeDetailSection) formFactory
//                .createEntityDetailSection(EntityDetailType.GEOGRAPHICAL_SCOPE,
//                        getConversationHolder(), parent, Section.TWISTIE | Section.COMPACT);
//
//        formFactory.createHorizontalSeparator(parent, SWT.BORDER);
//
//        ScopeRestrictionSection scopeRestrictionSection = (ScopeRestrictionSection) formFactory
//                .createEntityDetailSection(EntityDetailType.SCOPE_RESTRICTION,
//                        getConversationHolder(), parent, Section.TWISTIE | Section.COMPACT);
//
        formFactory.createHorizontalSeparator(parent, SWT.BORDER);

        TaxonomicScopeSection taxonomicScopeSection = (TaxonomicScopeSection) formFactory
                .createEntityDetailSection(EntityDetailType.TAXONOMIC_SCOPE,
                        getConversationHolder(), parent, Section.TWISTIE | Section.COMPACT);        

	    addPart(polytomousKeyDetailSection);
	    addPart(polytomousKeyNodeDetailSection);
//	    addPart(geographicalScopeDetailSection);
//	    addPart(scopeRestrictionSection);
	    addPart(taxonomicScopeSection);

	}
	
	/**
	 * create the UseRecordSection
	 * @param parent
	 */
	private void createUseRecordSection(RootElement parent) {
		destroySections();

		UseRecordDetailSection descriptionUseRecordSection = (UseRecordDetailSection) formFactory
				.createCdmDetailSection(DetailType.USE_RECORD,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(descriptionUseRecordSection);
		
		
	}

	/**
	 * createUserSection
	 * @param parent
	 */
	private void createUserSection(RootElement parent) {
		destroySections();

		UserDetailSection userDetailSection = (UserDetailSection) formFactory
				.createCdmDetailSection(DetailType.USER,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		GroupsByUserDetailSection groupByUserDetailSection = (GroupsByUserDetailSection) formFactory
				.createEntityDetailSection(EntityDetailType.GROUPS_BY_USER,
						getConversationHolder(), parent, Section.TWISTIE
								| Section.EXPANDED);

		addPart(userDetailSection);
		addPart(groupByUserDetailSection);
	}

	/**
	 * createTaxonRelationshipSection
	 * @param parent
	 */
	private void createTaxonRelationshipSection(RootElement parent) {
		destroySections();

		TaxonRelationshipDetailSection taxonRelationshipDetailSection = (TaxonRelationshipDetailSection) formFactory
				.createCdmDetailSection(DetailType.TAXON_RELATIONSHIP,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);

		ReferencedEntityDetailSection referencedEntityBaseDetailSection = (ReferencedEntityDetailSection) formFactory
				.createCdmDetailSection(DetailType.REFERENCED_ENTITY,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);
		addPart(taxonRelationshipDetailSection);
		addPart(referencedEntityBaseDetailSection);
	}

	/**
	 * @param rootElement
	 */
	private void createTermVocabularySection(RootElement parent) {
		destroySections();

		TermVocabularyDetailSection termVocabularyDetailSection = (TermVocabularyDetailSection) formFactory
				.createCdmDetailSection(DetailType.TERM_VOCABULARY,
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);
		addPart(termVocabularyDetailSection);
	}

	/**
	 * createDefinedTermSection
	 * @param parent
	 */
	private void createDefinedTermSection(RootElement parent) {
		destroySections();

		AbstractFormSection definedTermDetailSection = formFactory
				.createDefinedTermDetailSection(getInput().getClass(),
						getConversationHolder(), parent, this, Section.TWISTIE
								| Section.EXPANDED);
		addPart(definedTermDetailSection);
	}
}
